/*ce: (c) by Christian Ensel -- 17.6.2001
   under GPL -- see COPYING.TXT

   This little program is used to generate a proper models-header-file
   for the merlin-handwriting-recognition.

   input:  the original / manually modified / etc. header file is '#include'ed
           thus, the data is not read, but compiled in.
   output: writes a new header file - only based on the path-information
           of the original file. The rest is freshly calculated.

   How to change/add letters in the model:
           if you want to change or add a letter, edit/add the path in the
           INFILE. Make sure, that the number of values in the path and
           slope entries are equal (although the actual values of the slopes
           does not matter) and the letter is contained in patterns[].
           Then, recompile this file and run it. It writes the new correct
           header file.

   Hint:   Add ...\C\Merlin\h to your include-path.
           Compile in Merlin\C.
           Compiles fine with good ol' TC.
*/

#define OUTFILE "..\\H\\models.h"
#define INFILE <raw_mod.h>

/* define the following variable, if you want to reshape the paths (variable should usually be uncommented) */
#define CORRECT_PATH

#define POINTS_PER_LINE 5
#define SLOPES_PER_LINE 10



/* here comes the code */

#define FAR

#include <stdio.h>
#include <stdlib.h>
#include <pattern.h>
#include INFILE

#define  PI  3.1415926535897932386

#define DONEMARKER (-20000)

int fprintchar( FILE *f, unsigned char c)
{
    if ( c > 32 && c<='z' && c!='\\' && c!='\'')
        return fprintf(f, " '%c'", c );
    else
        return fprintf(f, "0x%02X", (int)c );

}


#define CE_REUSE_SLOPE_MEM
#include "patternf.c"

/* following function matches the one in filter.c */
#define NORM_SIZE   100

Pattern FAR *sizeNormFilter( Pattern FAR *p )
{
  int i, xmin, xmax, ymin, ymax;
  int delta;
  Point tmp;

  if ( p == NULL )
    return NULL;

  tmp = p->pv[ 0 ];
  xmin = xmax = tmp.x;
  ymin = ymax = tmp.y;

  for ( i = 1; i < p->nPoints; i++ ) {
    tmp = p->pv[ i ];
    if ( tmp.x < xmin )
      xmin = tmp.x;
    if ( tmp.y < ymin )
      ymin = tmp.y;
    if ( tmp.x > xmax )
      xmax = tmp.x;
    if ( tmp.y > ymax )
      ymax = tmp.y;
  }

  delta = max(xmax - xmin, ymax - ymin);
  xmin *= NORM_SIZE-1;
  ymin *= NORM_SIZE-1;

  for ( i = 0; i < p->nPoints; i++ ) {
    tmp = p->pv[i];
    p->pv[i].x = ((NORM_SIZE-1) * tmp.x - xmin) / delta;
    p->pv[i].y = ((NORM_SIZE-1) * tmp.y - ymin) / delta;
  }

  return p;
}



int main(int argc, char* argv[])
{
int i,j;
int nallpatterns=0;
int count;
int m,mi;
FILE *f;


/*  count all patterns */
for (i=0; i<nmodels; i++)
    nallpatterns += npatterns[i];


/****** calculate data from paths ******/
/* (throw away slope and pre-decission information and recalculate) */

for (i=0; i<nallpatterns; i++) {
    Pattern FAR *p = &(patterns[i]);
    #ifdef CORRECT_PATH
        sizeNormFilter(p);
    #endif
    calcSlopes(p);
    calcPreDecideInfo(p);
}


/****** output *** write new models.h in file OUTFILE ******/

/* f = stdout; */
f = fopen(OUTFILE,"wt");

/* header */
fprintf(f,"#ifndef MODELS_H\n"
          "#define MODELS_H\n\n");

/* write Paths - in the order of patterns[], marking the paths to avoid doublicate entries */
fprintf(f,"static Point FAR points[] = {\n");
count = 0;
for (i=0; i<nallpatterns; i++) {
    int startindex = count;
    Point *path = patterns[i].pv;
    if ( path[0].x == DONEMARKER )
        continue;
    fprintf(f, "/* Path:");
    fprintchar(f, patterns[i].ascii );
    fprintf(f, " (+%i) */",startindex);
    for (j=0; j<patterns[i].nPoints; j++) {
        count++;
        if (j%POINTS_PER_LINE==0)
            fprintf(f, "\n");
        fprintf(f, "{%2i,%2i}, ",path[j].x,path[j].y);
    }
    fprintf(f, "\n\n");
    path[0].x = DONEMARKER;
    path[0].y = startindex;
    path[1].x = (int)patterns[i].ascii;
}
fprintf(f, "};\n\n");


/* slopes */
fprintf(f, "static int FAR slopes[] = {\n ");
count = 0;
for (i=0; i<nallpatterns; i++) {
    int startindex = count;
    int *slope = patterns[i].s;
    if ( slope[0] == DONEMARKER )
        continue;
    fprintf(f, "/* Slope:");
    fprintchar(f, patterns[i].ascii );
    fprintf(f, " (+%i) */",startindex);
    for (j=0; j<patterns[i].nPoints; j++) {
        count++;
        if (j%SLOPES_PER_LINE==0)
            fprintf(f, "\n ");
        fprintf(f, "%4i, ",slope[j] );
    }
    fprintf(f, "\n\n");
    slope[0] = DONEMARKER;
    slope[1] = startindex;  /* we assume that there are at lest two points in a pattern */
}
fprintf(f, "};\n\n");


/*** patterns ***/
fprintf(f, "static Pattern FAR patterns[] = {\n"
           "/*** Model 0 ***/\n" );
m=0;  /* the current model-nr */
mi=0; /* the index in that model */

for (i=0; i<nallpatterns; i++) {
    /* mark beginning of new model */
    if (mi==npatterns[m]) {
        m++;
        fprintf(f, "\n/*** Model %i ***/\n",m);
        mi=1;
    }
    else {
        if (i%5==0 && i>0)    /* empty line every 5 patterns */
            fprintf(f, "\n");
        mi++;
    }
    /* the pattern */
    fprintf(f, "{ points + %4i, slopes + %4i, %2i, ",
               patterns[i].pv[0].y, patterns[i].s[1], patterns[i].nPoints );
    fprintchar(f, patterns[i].ascii );
    fprintf(f, ", %2i, %3i, %3i }",
               patterns[i].nPoints,  patterns[i].sev_x, patterns[i].sev_y ); /*, patterns[i].boxshape ); */
    if (i%10==0)
       fprintf(f, " /* %i */",i);
    if ( patterns[i].pv[1].x != (int)(patterns[i].ascii) ) {
       fprintf(f, " /*");
       fprintchar(f, (unsigned char)(patterns[i].pv[1].x));
       fprintf(f, " has same pattern */" );
    }
    fprintf(f, ",\n");
}
fprintf(f, "};\n\n");

/*** models ***/
fprintf(f, "Pattern FAR *models[] = {\n  patterns + 0,\n" );
count = 0;
for (i=0; i<nmodels; i++) {
    count += npatterns[i];
    fprintf(f, "  patterns + %i,\n", count);
}
fprintf(f, "};\n\n");

/* npatterns */
/* fprintf(f, "static int npatterns[] = {"); */
fprintf(f, "int npatterns[] = {");
for (i=0; i<nmodels; i++)
    fprintf(f, " %i,", npatterns[i]);
fprintf(f, " 0, };\n\n");

/* nmodels */
fprintf(f, "static int nmodels = %i;\n\n", nmodels);

/* trailer */
fprintf(f, "#endif\n");
fclose(f);

return 0;
}



