#include <stdrom.h>

#include "define.h"
#include "libc.h"
#include "l_define.h"
#include "l_libc.h"

#include "md.h"

#include "mm.h"
#include "event.h"
#include "mod_symb.h"

#include "util.h"

#include "modebut.h"

#include "ml_help.h"
#include "menubar.h"
#include "loadsave.h"

#include "permanent.h"
#include "filter.h"
#include "keywords.h"


#ifdef POLISH
#include "acctbl-pl.h"
#else
#include "acctbl.h"
#endif

static void handleMenuBar();
int editParamWindow(int p);

#define HEAP_SIZE  (20*1024)

/*ce4 #define OBJ_MYHARDICON 0x8002 */

#define ML_HEIGHT_FULL   144
#define ML_HEIGHT_SMALL  72

/*** Designation of text area ***/
#define M_ST_X 2 /* st_x */
#define FRAME_ST_X M_ST_X-2

#define M_ST_Y 15 /* st_y */
#define M_ED_X (M_ST_X+ML_WIDTH+2) /* ed_x */
#define M_IT_Y 9 /* it_y */
/* #define M_MAXG ((ML_HEIGHT+2)/(M_IT_Y)) /* MAXGYO */
#define M_MAXG ((merlinTch[0].y2-merlinTch[0].y1+1)/(m_in_tp.it_y)) /* MAXGYO */
/*** Scroll bar setting ***
#define SCR__Y M_ST_Y /* Scroll bar position *
#define SCR__SIZE M_IT_Y*M_MAXG /* Height of scroll bar *
#define SCR__DSP M_MAXG /* Number of display records *
*/

static SHOT_INF screen; /* the buffer in screen is also used several times as temporary (string) buffer!! */


/***** messages *****/
typedef enum {  MSG_SAVEPAGE_FAILED,
                MSG_LPART_SAVED,
                MSG_P2LARGE,
                MSG_ERROR_DEL_P,
                MSG_NONXTP,
                MSG_COUNTER_NOTSTARTED,
                MSG_LOADINGLAST,
                MSG_LOADING1ST,
                MSG_ERROR_BUTSAVED,
                MSG_DELETE_FOREVER,
                MSG_DELETED,
                MSG_AUTOCOPY,
                MSG_MUST_CHOOSE
} t_GeneralMsgNr;  /* type (enum) not used, only constants */

char *generalMsg[] = {
     "Saving current page\rfailed!",
     "Lower part saved!",
     "Pages too large!\rCannot merge.",
     "Error deleting\rmerged page!",
     "No next page found!",
     "Time-diff counter\r"
                     "not started, yet!\r\r"
                     "To do so, insert normal\r"
                     "time via time menu.",
     "loading last",
     "loading 1st",
     "File read error!\r(page saved)",
     "Delete forever?",
     "File deleted!",
     "Auto-Copy selection\rto clipboard.",
     "New Contact needs\ra category!"
};


typedef enum {  MSG_SELCATEGORY,
                MSG_HASNOTITLE,
                MSG_SAVEQ,
                MSG_STILLNEW,
                MSG_DELETE_NEW,
                MSG_DELETE
} t_dmDMsgNr;  /* type (enum) not used, only constants */

char *datamodeDependentMsg[] = {
     "Memo Category?",
     "Memo has no title!",
     "Save Memo?",
     "Memo is new,\rsave it first!",
     "Delete new Memo?",
     "Delete Memo?",
#ifdef TEST2
     "Contact Category?",
     "Name field empty!",
     "Save Contact?",
     "Contact is new,\rsave it first!",
     "Delete new Contact?",
     "Delete Contact?"
#endif
};


#define GMSG(M) generalMsg[M]
#ifdef TEST2
  #define DMSG(M) datamodeDependentMsg[ (p_pvmerlin_permanent_data->currentdatamode == CONTACTMODE) ? (M)+MSG_DELETE+1 : (M) ]
#else
  #define DMSG(M) datamodeDependentMsg[M]
#endif

/*--- parameters and permanent stuff ---*/

#include "params.h"

/*old: 1st entry is menu-title of parameter-sub-menu
  and textentry of main parameter menu
  (now two different arrays) */

static byte* param_names[NR_PARAMS] =
  { "Writing style",
    "Selection threshold",
    "Character recognition",
    "Text font",
    "Line spacing",
    "Show 'return'",
    "Complete words",
    "Start with",
    "Save on exit (esc)",
    "Action wheel function",
#ifdef TEST
    "Time format",
    "Input buffer",
    "Input buffer delay",
#endif
#ifdef ALLOWDEBUGCODE
    "Debug"
#endif
    };

static byte* param_choices[NR_PARAMS] =
  { "normal\xfe" "slanted\xff" ,                              /* PARAM_WRITING_STYLE,            */
    "normal\xfe" "very short\xfe" "short\xfe" "long\xff" ,    /* PARAM_SELECTION_TIME_THRESHOLD, */
    "always on\xfe" "non-keyboard mode only\xff",             /* PARAM_MERLIN_WITH_KEYBOARD,     */
#ifdef TEST
    "normal\xfe" "bold\xfe" "non-proportional\xff",           /* PARAM_FONT,                     */
#else
    "normal\xfe" "bold\xff",                                  /* PARAM_FONT,                     */
#endif
    "normal\xfe" "narrow\xff" ,                               /* PARAM_LINESPACING,              */
    "off\xfe"    "on\xff",                                    /* PARAM_SHOWRET,                  */
    "on (loaded and written words)\xfe"  "on (written words only)\xfe"  "off\xff",  /* PARAM_KEYWORDS, */
#ifdef TEST2
    "last file\xfe" "load dialog\xfe" "empty Memo\xff",       /* PARAM_START_WITH,               */
#else
    "last Memo\xfe" "load dialog\xfe" "empty Memo\xff",       /* PARAM_START_WITH,               */
#endif
    "ask\xfe" "always\xff" ,                                  /* PARAM_SAVE_ON_EXIT,             */
#ifdef TEST2
    "paragraph&page\xfe" "page\xfe" "file\xfe" "time\xff" ,   /* PARAM_LPS,                      */
#else
    "paragraph&page\xfe" "page\xfe" "memo\xfe" "time\xff" ,   /* PARAM_LPS,                      */
#endif
#ifdef TEST
    "#) delta total avg\xfe" "#) total\xfe" "total (delta)\xfe" "#)\xfe" "total\xfe" "delta\xff",  /* PARAM_TIMEFORMAT,         */
    "at >500 chars\xfe" "at >200 chars\xfe" "always on\xfe" "off\xff",  /* PARAM_CHBUF,         */
    "normal\xfe" "very short\xfe" "long\xff",                 /* PARAM_CHBUFDELAY,         */
#endif
#ifdef ALLOWDEBUGCODE
    "off\xfe" "pattern\xfe" "pre-proc-info\xfe" "keywords\xfe" "all\xff" /* PARAM_DEBUG,         */
#endif
  };

/* return values of main parameter menu (must be sequential, starting from 0!) */
/*old (values must go from 0 to NR_PARAMS-1 anyway...)
static byte param_rtv[NR_PARAMS+1] =
  { PARAM_WRITING_STYLE,
    PARAM_SELECTION_TIME_THRESHOLD,
    PARAM_MERLIN_WITH_KEYBOARD,
    PARAM_FONT,
    PARAM_LINESPACING,
    PARAM_SHOWRET,
    PARAM_KEYWORDS,
    PARAM_START_WITH,
    PARAM_SAVE_ON_EXIT,
    PARAM_LPS,
    PARAM_DEBUG,
    0xff};
*/

/*buffer for menu*/
/*#define PARAM_MENU_CHARS (32+1) /* Max length of one param-menu line *
 misuse screen-copy-buffer instead ;-)
static byte param_menu[NR_PARAMS*(PARAM_MENU_CHARS+1)+1];
*/
static byte* param_menu = screen.g_buf;


#ifdef TEST2
  #ifdef ALLOWDEBUGCODE
    #define PVM_PERMANENT_FILENAME "PVMerlin.d40" /* code size too large! */
  #else
    #define PVM_PERMANENT_FILENAME "PVMerlin.v40"
  #endif
  static byte catbaksave=-1;  /* saves original categoryBak at the beginning of the secret mode */
#else
  #ifdef TEST
    #define PVM_PERMANENT_FILENAME "PVMerlin.v32"
  #else
    #define PVM_PERMANENT_FILENAME "PVMerlin.v30"
  #endif
#endif

#ifdef TEST2
  #define WIDEICON ( inSecretMode() || p_pvmerlin_permanent_data->searchMode )
#else
  #define WIDEICON ( inSecretMode() )
#endif

s_pvmerlin_permanent_data* p_pvmerlin_permanent_data = NULL;

int setupParams();
void enterString(char *txt, int notifykeyword);

static byte catsave=-1;  /* saves original category at the beginning of the secret mode */

/* --- */

static TCHTBL InptTch[3]; /* Text touch area table */
static u8 far heap[HEAP_SIZE];
static int x,y;
static byte curch;

static int keyboardmode;

bool MERLINACTIVE() {
    return (keyboardmode==0 || PARAM(PARAM_MERLIN_WITH_KEYBOARD)==P_MWKEYBOARD_ON);
}
/*#define MERLINACTIVE (keyboardmode==0 || PARAM(PARAM_MERLIN_WITH_KEYBOARD)==P_MWKEYBOARD_ON)*/

static TCHTBL far merlinTch[2]; /* {
       /* { MERLIN_TCH(M_ST_X+1, M_ST_Y+1, OBJ_MERLIN) }, */
/*       { M_ST_X+1, M_ST_Y+1, M_ST_X + ML_WIDTH, M_ST_Y + ML_HEIGHT_FULL, ML_ACTMASK, OBJ_MERLIN, 0 },
       { 0,0,0,0,0,OBJ_END,0 }
};
*/

/*TCHTBL far actionWheelTch[];  /*ceV3.1 nach util.c verlegt */

/*ce original line: { MERLIN_TCH(40, 160-ML_HEIGHT, OBJ_MERLIN) }, */

/*ce textwidget stuff (taken out of eventloop()) */
static TCHSTS tsts;
static TXTP m_in_tp;       /* Declaration of structure for text input*/
static T_SCR_POS m_in_scb; /* Scroll bar position information */

static word currentfp; /* ==NEWFILE for new file; otherwise a valid pointer to flush-memory */

static char textbuffer[2049];


static bool restart;


/***/
/* handle memo-changed state */

static bool memo_changed=HALF;  /* HALF means that Icon it is going to be drawn at the next setMemo...-call in any case */

bool memoHasChanged();
static void drawFileState();
void setMemoChanged();
static void setMemoFresh();


/***/

/*checks whether newchar is an accent for prevchar;
  if yes, returns charcter code of lastAccentChar
  if no, returns 0 */
int checkAccentable(int prevchar, int newchar) {
    char *pc;
    int i_ac, i_pc;

    pc = strchr( accent, newchar );
    if ( pc!=NULL ) { /* new char is an accent  */
        i_ac = pc - accent;
        pc = strchr( simplechars, prevchar );
        if ( pc!=NULL ) { /* prev. character is accentable */
            i_pc = pc - simplechars;
            return accentedchars[i_ac][i_pc];  /* set new char */
        }
    }
    return 0;
}

/***/

/*ce apply a change of parameter p.
 returns whether the textwidget has to be updated */
int applyParam(int p)
{
    int screen_changed=0;
    byte pval = PARAM(p);
    switch (p) {
        case PARAM_WRITING_STYLE:
            if ( pval == P_CURSIVE )
                setSlantedFilterStrength(3);
            else
                setSlantedFilterStrength(9); /* off */
            break;
        case PARAM_SELECTION_TIME_THRESHOLD:
            mlSetPassTimeThreshold(pval);
            break;
        case PARAM_MERLIN_WITH_KEYBOARD:
            if (keyboardmode==1)
                restart = TRUE;
            break;
        case PARAM_LINESPACING:
            m_in_tp.it_y   = M_IT_Y-pval;     /* Text display line spacing (Y) */
            screen_changed = 1;
            if (keyboardmode==0) {
                screen.y_size = 126+(pval<<1);
            } else {
                screen.y_size = ML_HEIGHT_SMALL;
            }
            m_in_tp.MAXGYO = M_MAXG; /* Number of text display lines */
            LibLineClr(M_ST_X-1,merlinTch[0].y2-1,ML_WIDTH,1); /* dirty trick to clean away some dust left on the screen */
            break;
        case PARAM_FONT:
            m_in_tp.font = pval; /*!! assumes that IB_PFONT1==0 and IB_PFONT2==1 and IB_PFONT3==2 */
            screen_changed = 1;
            break;
        case PARAM_SHOWRET:
            m_in_tp.rtnen = pval;
            screen_changed = 1;
            break;
        case PARAM_KEYWORDS:
            addKeywords(m_in_tp.txbf);
            break;
    }
    return screen_changed;
}

/* draw Icon of PVMerlin incl. file-state */
/* returns width in pixels used on the screen */
int drawMainIcon()
{
   int x=15;
#ifdef TEST2
   if ( p_pvmerlin_permanent_data->searchMode ) {
        LibClrBox(0,0,14,11);
        LibPutFarData( 0, 0, 96);
        LibPutDispBox(0,0,12,11);
   }
   else 
#endif
   if ( inSecretMode() ) {
        LibClrBox(0,0,14,10);
        /*LibClrBox( 0,0, 12,10);*/
        LibPutFarData( 0, 0, 78);
        LibPutDispBox(0,0,12,10);
    }
    else {
        LibClrBox( 0,0, 10,10);
        LibGdsLine( 7, 1, 7, 8 );
        LibGdsLine( 7, 1, 0, 8 );
        LibGdsLine( 0, 9, 7, 9 );
        LibGdsLine( 7, 3, 2, 8 );
        LibGdsLine( 7, 5, 4, 8 );
        x=11;
    }
    drawFileState();
    return x;
}

/* draw program name (starting at x-coordinate x, returns last used x-coordinate +1
   if (isLong) print the fullname, otherwise keep it short )*/
int drawProgramName(int x, int isLong) {
    LibPutProFont(IB_PFONT3, 'P', x, 1 );
    x += LibGetProFontSize(IB_PFONT3, 'P' )-2;
    LibPutProFont(IB_PFONT3, 'V', x, 1 );
    x += LibGetProFontSize(IB_PFONT3, 'V' )-1; /* 0 */
    LibPutProFont(IB_PFONT3, 'M', x, 1 );
    x += LibGetProFontSize(IB_PFONT3, 'M' )-1;

    if (isLong) {
        LibPutProFont(IB_PFONT3, 'e', x, 1 );
        x += LibGetProFontSize(IB_PFONT3, 'e' )-1;
        LibPutProFont(IB_PFONT3, 'r', x, 1 );
        x += LibGetProFontSize(IB_PFONT3, 'r' )-2;
        LibPutProFont(IB_PFONT3, 'l', x, 1 );
        x += LibGetProFontSize(IB_PFONT3, 'l' )-0;
        LibPutProFont(IB_PFONT3, 'i', x, 1 );
        x += LibGetProFontSize(IB_PFONT3, 'i' )-0;
        LibPutProFont(IB_PFONT3, 'n', x, 1 );
        x += LibGetProFontSize(IB_PFONT3, 'n' );
        if ( !WIDEICON )
            x++;
#ifdef TEST2
        LibPutProFont(IB_PFONT3, '4', x, 1 );
        x += LibGetProFontSize(IB_PFONT3, '4' )-1;
#else
        LibPutProFont(IB_PFONT3, '3', x, 1 );
        x += LibGetProFontSize(IB_PFONT3, '3' )-1;
#endif
    }
    return x;
}

static int mainTitleX;
static int mainTitleX2;

#define TITLE_IS_LONG (keyboardmode==0 || !MERLINACTIVE())

/*ce redraw screen-stuff like frame, title etc. */
static void drawBackground()
{
    LibClrDisp();

    mainTitleX = drawMainIcon()+1;
    mainTitleX2 = drawProgramName( mainTitleX, TITLE_IS_LONG );

        /*x += LibGetProFontSize(IB_PFONT3, '3' )+0;
        LibPutProStr(IB_PFONT1, x, 4, "beta", 159 );*/

    /* Frame */
    LibGdsLine(FRAME_ST_X, M_ST_Y-2, 159, M_ST_Y-2 );
    LibGdsLine(FRAME_ST_X, M_ST_Y-1, FRAME_ST_X, M_ST_Y+M_MAXG*m_in_tp.it_y-1 );
    LibGdsLine(FRAME_ST_X, M_ST_Y+M_MAXG*m_in_tp.it_y, M_ED_X+SCR_XSIZE+5, M_ST_Y+M_MAXG*m_in_tp.it_y );

    mlPaint();
    LibPutDisp();
}


#ifdef TEST2
byte prevditp[ML_HEIGHT_FULL/8+1]; /* remember which contact icons are currently painted on the screen*/
int privateContactsIcons[] = { 274, 282, 283, 284, 285, 279, 275, 276, 277, 280, 281, 278, 286 };
#endif

int myTxtDspC(TXTP *p_mintp)
{
    if(LibTxtDspC(p_mintp)==TRUE) { /* Display during text input */
#ifdef TEST2
        int l;
        if ( p_pvmerlin_permanent_data->currentdatamode == CONTACTMODE ) {
            /* display contact icons */
            for (l=1; l<=m_in_tp.MAXGYO; l++) {
                if ( m_in_tp.ditp[l] != prevditp[l] || prevditp[l]==0xfe) {
                    prevditp[l] = m_in_tp.ditp[l];
                    if (prevditp[l]<0xff)
                        /*LibPutProFont(IB_PFONT2, '0'+m_in_tp.ditp[l], FRAME_ST_X+2, (l-1)*m_in_tp.it_y + M_ST_Y +1 ); */
                        LibPutFarData(FRAME_ST_X+2, (l-1)*m_in_tp.it_y + M_ST_Y,
                            CAT_ISBUSINESSCONTACT(p_pvmerlin_permanent_data->currentcat) ? 274+prevditp[l] :
                            ( p_pvmerlin_permanent_data->currentcat==1 ? privateContactsIcons[prevditp[l]] :
                              287 + prevditp[l] ) );
                    else
                        LibClrBox(FRAME_ST_X+2, (l-1)*m_in_tp.it_y + M_ST_Y,
                            12, m_in_tp.it_y );
                }
            }
        }
#endif
        LibPutDisp();
        return TRUE;
    }
    return FALSE;
}


static void merlinTxtInit(TXTP *p_mintp, int cp)
{
    m_in_tp.st_x = M_ST_X;   /* Start coordinate (X) of text display */
#ifdef TEST2
    if (p_pvmerlin_permanent_data->currentdatamode == CONTACTMODE) {
        LibClrBox(m_in_tp.st_x, m_in_tp.st_y, 13, M_MAXG*m_in_tp.it_y );
        m_in_tp.st_x += 13;
    }
#endif
    LibTxtInit(p_mintp);

    /*p_mintp->s_pos->x += 1;*/
    if (keyboardmode==0) {
        p_mintp->s_pos->y += 60;
        p_mintp->s_pos->size -= 60+7;
    }
    else {
        p_mintp->s_pos->y += 23;
        p_mintp->s_pos->size -= 23;
        LibDispKey();
    }
    p_mintp->cp = cp;

#ifdef TEST2
    {
      int i;
      for (i=0; i<=m_in_tp.MAXGYO; i++)
        prevditp[i]=0xfe; /* nothing drawn, yet */
    }
#endif
    myTxtDspC(p_mintp);
}

static void initApp()
{
    mm_create_heap(heap, HEAP_SIZE);
    LibClrDisp();
}


static void clearTextBuffer()
{
    /* to update display and cursor, each call to this function must
       be followed by: merlinTxtInit(&m_in_tp, 0); */
#ifdef TEST2
    if ( p_pvmerlin_permanent_data->currentdatamode == CONTACTMODE ) {
        int i;
        for (i=0; i<13-1; i++) /* there are 13 entries in one contact entry */
            m_in_tp.txbf[i]=0xfe;
        m_in_tp.txbf[12]=0;
    }
    else
#endif
        m_in_tp.txbf[0]=0;
    currentfp = NEWFILE;
    /* p_pvmerlin_permanent_data->currentcat = -1; ceV4 allow to keep last category */
    p_pvmerlin_permanent_data->cursorpos = 0;
    m_in_tp.cp = 0;
    resetKeywordprefix();
    setMemoFresh();
}




static setPermanentTextParams()
{
    int i;
    /* get full title */
    i=0;
    if (currentfp!=NEWFILE) {
        while ( i<MAX_TITLE_LEN && m_in_tp.txbf[i]!=0 && m_in_tp.txbf[i]!='\r') {
            p_pvmerlin_permanent_data->fulltitle[i] = m_in_tp.txbf[i];
            i++;
        }
    }
    p_pvmerlin_permanent_data->fulltitle[i] = 0;
    p_pvmerlin_permanent_data->cursorpos = m_in_tp.cp;
}

/*ce
  if used in connection with savetext, this should be called after savetext
  to make sure the title gets saved, too. */
static void saveParams()
{
    if ( !inSecretMode() ) {
        setPermanentTextParams();
        permanentSave();
    }
    else {
        int cat = p_pvmerlin_permanent_data->currentcat;
#ifdef TEST2
        int catBak = p_pvmerlin_permanent_data->currentcatBak;
        p_pvmerlin_permanent_data->currentcatBak = catbaksave;
#endif
        p_pvmerlin_permanent_data->currentcat = catsave;
        permanentSave();
        p_pvmerlin_permanent_data->currentcat = cat;
#ifdef TEST2
        p_pvmerlin_permanent_data->currentcatBak = catBak;
#endif
    }

}

static void initNewTxtContent() {
    addKeywords(m_in_tp.txbf);
    merlinTxtInit(&m_in_tp, 0);
    saveParams(); /* remember currently edited file-title */
    setMemoFresh();
}    



/*** init new Memo ***/

static void startNewMemo()
{
    clearTextBuffer();
#ifdef TEST2
    if ( p_pvmerlin_permanent_data->currentdatamode == CONTACTMODE) {
        p_pvmerlin_permanent_data->currentcat = selectcategory(p_pvmerlin_permanent_data->currentcat, DMSG(MSG_SELCATEGORY) );
        if (p_pvmerlin_permanent_data->currentcat==0xff) { /* esc also chooses personal category */
            p_pvmerlin_permanent_data->currentcat = 1;
        }
    } else
#endif
        p_pvmerlin_permanent_data->currentcat = -1;
    initNewTxtContent();
}


/*** doSave? ***/

typedef enum { DOSAVE_NO, DOSAVE_BREAK, DOSAVE_YES} t_dosave;

t_dosave doSaveDialog()
{
    int sel = SelWindowExt2(-1, -1, DMSG(MSG_SAVEQ), 0, "yes\x0FEno\x0FF", NULL );
    if (sel==0)
        return DOSAVE_YES;
    if (sel==1)
        return DOSAVE_NO;
    return DOSAVE_BREAK;
}


/*ce: should be called before each operation that may delete/exchange the
 current memo-text
 Allows user to cancel or to save text before continuing */
/* returns whether user wants to continue */
bool checkMemoSavedContinue() {
    if ( memoHasChanged() ) {
        t_dosave dosave = doSaveDialog();
        if ( dosave == DOSAVE_BREAK )
            return FALSE;
        else if ( dosave == DOSAVE_YES) {
            int saveok = savetext( m_in_tp.txbf, &currentfp, &(p_pvmerlin_permanent_data->currentcat) );
            if ( saveok != TRUE ) {
                if (saveok==FALSE)
                    LibPutMsgDlg(GMSG(MSG_SAVEPAGE_FAILED));
                return FALSE;
            }
            else
                setMemoFresh();
        }
    }
    return TRUE;
}



/*********** titlebuttons (ce) **************/

#define NRTITLEBUTTONS 11
#define TITLEBUTTON_MODECHANGE 0  /* now used for about box, no mode-switch at all... but now: V4 implemented the mode switch */
#define TITLEBUTTON_PAGE   1
#define TITLEBUTTON_TIME   2
#define TITLEBUTTON_CALC   3
#define TITLEBUTTON_DEL    4
#define TITLEBUTTON_HELP   5
#define TITLEBUTTON_TOOLS  6
#define TITLEBUTTON_PGUP   7
#define TITLEBUTTON_PGDN   8
#define TITLEBUTTON_KEYB   9
#define TITLEBUTTON_TITLE  10
#define TITLEBUTTON_OBJ_BASE 0x8080

static TCHTBL far titleButTchTbl[] = {
       {   0, 0,  11, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_MODECHANGE, 0x0000 },
       {  69, 0,  82, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_PAGE, 0x0000 },
       {  85, 0,  96, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_TIME, 0x0000 },
       {  99, 0, 107, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_CALC, 0x0000 },
       { 111, 0, 129, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_DEL, 0x0000 },
       { 131, 0, 139, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_HELP, 0x0000 },
       { 140, 0, 158, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_TOOLS, 0x0000 },
       { 158-MODEBUTTON_XSIZE, 13, 159, 23, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_PGUP, 0x0000 },
       { 158-MODEBUTTON_XSIZE, 24, 159, 36, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_PGDN, 0x0000 },
       { 160-12, 160-9, 159, 159, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_KEYB, 0x0000 },
       { 12, 0, 66, 11, ACT_ICON, TITLEBUTTON_OBJ_BASE+TITLEBUTTON_TITLE, 0x0000 },
       { 0,0,0,0,ACT_NONE,OBJ_END,0x0000 }
};

static T_ICON far titleButIcons[] = {
     {&titleButTchTbl[TITLEBUTTON_MODECHANGE],NULL,NULL,0x03},
     {&titleButTchTbl[TITLEBUTTON_PAGE], NULL,NULL,0x03},
     {&titleButTchTbl[TITLEBUTTON_TIME], NULL,NULL,0x03},
     {&titleButTchTbl[TITLEBUTTON_CALC], NULL,NULL,0x01},
     {&titleButTchTbl[TITLEBUTTON_DEL],  NULL,NULL,0x02},
     {&titleButTchTbl[TITLEBUTTON_HELP], NULL,NULL,0x03}, /* 01 */
     {&titleButTchTbl[TITLEBUTTON_TOOLS],NULL,NULL,0x02},
     {&titleButTchTbl[TITLEBUTTON_PGUP], NULL,NULL,0x03},
     {&titleButTchTbl[TITLEBUTTON_PGDN], NULL,NULL,0x03},
     {&titleButTchTbl[TITLEBUTTON_KEYB], NULL,NULL,0x03},
     {&titleButTchTbl[TITLEBUTTON_TITLE],NULL,NULL,0x03}
};


void titleButtonsInit()
{
    int x,y;

    /* help+tools+calc+time+del-buttons */
    LibPutFarData(160-30, titleButTchTbl[TITLEBUTTON_TOOLS].y1, 131);
    LibPutFarData(titleButTchTbl[TITLEBUTTON_TIME].x1-2, titleButTchTbl[TITLEBUTTON_TIME].y1, 133); /* time+calc+(too)large button */
    x = titleButTchTbl[TITLEBUTTON_TOOLS].x1;
    y = titleButTchTbl[TITLEBUTTON_TOOLS].y1;
    LibLineClr( x, y, 1, 12 );
    LibDotOff( x+1, y+11 );

    x = titleButTchTbl[TITLEBUTTON_HELP].x1;
    y = titleButTchTbl[TITLEBUTTON_HELP].y1;
    LibBox( x-2, y+1, 2, 9, 1 );
    LibDotOff( x-1, y );
    LibLineClr( x, y, 1, 12 );
    LibDotOff( x+1, y+11 );

    /* PgUp/PgDn */
    LibPutBoxSub( titleButTchTbl[TITLEBUTTON_PGUP].x1, titleButTchTbl[TITLEBUTTON_PGUP].y1,
                  titleButTchTbl[TITLEBUTTON_PGDN].x2-1, titleButTchTbl[TITLEBUTTON_PGDN].y2-1, IB_GDS_OVER );
    LibGdsLine( titleButTchTbl[TITLEBUTTON_PGDN].x2, titleButTchTbl[TITLEBUTTON_PGUP].y1+1,
                  titleButTchTbl[TITLEBUTTON_PGDN].x2, titleButTchTbl[TITLEBUTTON_PGDN].y2 );
    LibGdsLine( titleButTchTbl[TITLEBUTTON_PGUP].x1+1, titleButTchTbl[TITLEBUTTON_PGDN].y2,
                  titleButTchTbl[TITLEBUTTON_PGDN].x2, titleButTchTbl[TITLEBUTTON_PGDN].y2 );
    LibPutFarData( titleButTchTbl[TITLEBUTTON_PGUP].x1+2, titleButTchTbl[TITLEBUTTON_PGUP].y1+2, 257 );
    LibPutFarData( titleButTchTbl[TITLEBUTTON_PGDN].x1+2, titleButTchTbl[TITLEBUTTON_PGDN].y1+2, 184 );
    LibGdsLineMesh( titleButTchTbl[TITLEBUTTON_PGDN].x1+1, titleButTchTbl[TITLEBUTTON_PGDN].y1,
                    titleButTchTbl[TITLEBUTTON_PGDN].x2-1, titleButTchTbl[TITLEBUTTON_PGDN].y1 );

    /* help-button-text */
    LibPutProFont(IB_PFONT1, '?', x+1, y+2 );

    /* del-button-text */
    LibPutProStr(IB_PFONT1, titleButTchTbl[TITLEBUTTON_DEL].x1+3, titleButTchTbl[TITLEBUTTON_DEL].y1+2, "Del", 20 );

    /* page-button */
    x = titleButTchTbl[TITLEBUTTON_PAGE].x1;
    y = titleButTchTbl[TITLEBUTTON_PAGE].y1;
    LibGdsBox( x+2, y+2, titleButTchTbl[TITLEBUTTON_PAGE].x2,   titleButTchTbl[TITLEBUTTON_PAGE].y2 );
    x += 2;
    LibPutProFont(IB_PFONT1, 'p', x, y );
    x += LibGetProFontSize(IB_PFONT1, 'p' )-1;
    LibPutProFont(IB_PFONT1, 'g', x, y );
    LibGdsBox( titleButTchTbl[TITLEBUTTON_PAGE].x1,   y, titleButTchTbl[TITLEBUTTON_PAGE].x2-2, titleButTchTbl[TITLEBUTTON_PAGE].y2-2 );

    /* Tools-button-text */
    x = titleButTchTbl[TITLEBUTTON_TOOLS].x1+2;
    y = titleButTchTbl[TITLEBUTTON_TOOLS].y1+2;
    LibPutProFont(IB_PFONT1, 'T', x, y );
    x += LibGetProFontSize(IB_PFONT1, 'T' )-1;
    LibPutProFont(IB_PFONT1, 'o', x, y );
    x += LibGetProFontSize(IB_PFONT1, 'o' )-1;
    LibPutProFont(IB_PFONT1, 'o', x, y );
    x += LibGetProFontSize(IB_PFONT1, 'o' )-1;
    LibPutProFont(IB_PFONT1, 'l', x, y );

    /* keyboard-button*/
    if (keyboardmode==0) {
        LibPutFarData(titleButTchTbl[TITLEBUTTON_KEYB].x1, titleButTchTbl[TITLEBUTTON_KEYB].y1, 302);
        LibLineClr(titleButTchTbl[TITLEBUTTON_KEYB].x1, titleButTchTbl[TITLEBUTTON_KEYB].y1, 11, 1);
    }

    /* update display */
    x = titleButTchTbl[0].x1;
    y = titleButTchTbl[0].y1;
    LibPutDispBox( x, y, titleButTchTbl[NRTITLEBUTTONS-1].x1 - x, titleButTchTbl[NRTITLEBUTTONS-1].y1 - y);

    /* setup touch table */
    LibTchStackPush( titleButTchTbl );
}

/* check whether title is valid for a multi-page document (title is not empty) */
bool checktitleok(const char *c)
{
    while ( *c==' ' )  /* skip spaces */
      c++;
    if (IS_TITLE_ENDCHAR(*c))
        return FALSE;
    return TRUE;
}

/* copies the title from *txtbuf into the buffer *title */
void getTitle( char *txtbuf, char *title) {
    /* read current title, but not further than one of the end_chars (,-,:,\r,0x0 */
    if (IS_TITLE_ENDCHAR(txtbuf[0])) { /*(*txtbuf==0 || *txtbuf=='\r' || *txtbuf=='(' || *txtbuf=='-' || *txtbuf==':')*/
        *title = 0;
        return;
    }
    while ( !IS_TITLE_ENDCHAR(*txtbuf) )/*(*txtbuf!=0 && *txtbuf!='\r' && *txtbuf!='(' && *txtbuf!='-' && *txtbuf!=':') */
        *title++ = *txtbuf++;
    /* clean away spaces at the end*/
    title--;
    while ( *title==' ' )
        title--;
    title++;
    *title = 0;
}

static byte* page_menu = "split page at cursor\xfe" "merge with next page\xfe" "goto page\xff";
static byte* page_menu2 = "goto page\xff";
/*static byte page_menu_rtv[] = { 0, 1, 0xff };*/

static void pageMenu() {
/*    SLW2_ST slw;
    slw.x=40;
    slw.y=10;
    slw.xs=110;
    slw.ity=9;
    slw.np=0;
    slw.t_xs=4;
    slw.rtv=page_menu_rtv;
    slw.cmnt=page_menu; */
    int sel;
#ifdef TEST2
    if ( p_pvmerlin_permanent_data->currentdatamode == CONTACTMODE)
        sel = SelWindowExt2(65, 10, NULL, 0, page_menu2, NULL ) + 2;
    else
#endif
        sel = SelWindowExt2(40, 10, NULL, 0, page_menu, NULL );
    switch ( sel ) {
    case 0: /* split page */
        if ( checktitleok( m_in_tp.txbf ) ) {
            int titlelen;
            /* put lower page in buffer */
            getTitle( m_in_tp.txbf, screen.g_buf); /* misuse screen-copy-buffer for title-buffer */
            titlelen = strlen(screen.g_buf);
            if ( titlelen > m_in_tp.cp ) /* cursor in title: split behind title */
                m_in_tp.cp = titlelen;
            strcat( screen.g_buf, "\r");
            strcat( screen.g_buf, m_in_tp.txbf + m_in_tp.cp ); /* append text of lower page */
            /* save lower page */
            if ( savetext( screen.g_buf, &currentfp, &(p_pvmerlin_permanent_data->currentcat) ) == TRUE ) {
                word upperpagefp = NEWFILE;
                LibPutMsgDlg3(GMSG(MSG_LPART_SAVED));
                /* save upper page */
                m_in_tp.txbf[ m_in_tp.cp ] = 0;
                if (savetextAt( m_in_tp.txbf, &upperpagefp, &(p_pvmerlin_permanent_data->currentcat), currentfp ) ) {
                    setMemoFresh();
                    LibWinIcnMsg(ICON_OK,341,1); /* Data stored! */
                }
                /* continue work on upper page */
                merlinTxtInit(&m_in_tp, m_in_tp.cp );
            }
        }
        else
            LibPutMsgDlg(DMSG(MSG_HASNOTITLE));
        break;
    case 1: /* merge pages */
        if ( checktitleok( m_in_tp.txbf ) )  {
            word nextpagefp = currentfp;
            getTitle( m_in_tp.txbf, screen.g_buf); /* misuse screen-copy-buffer for title- and text-buffer */
            if ( loadpage( screen.g_buf, screen.g_buf, SEARCH_NEXT_PAGE,
                           &nextpagefp, &(p_pvmerlin_permanent_data->currentcat) ) ) {
                int l = strlen(m_in_tp.txbf);
                if ( l + strlen(screen.g_buf) +1 >= 2048 ) {
                    LibPutMsgDlg(GMSG(MSG_P2LARGE));
                    break;
                }
                m_in_tp.txbf[l++]='\r';
                m_in_tp.txbf[l]=0;
                strcpy( m_in_tp.txbf+l, screen.g_buf );
                addKeywords(m_in_tp.txbf+l);
                merlinTxtInit(&m_in_tp, l );
                if (!deletefile(nextpagefp))
                    LibPutMsgDlg(GMSG(MSG_ERROR_DEL_P));
                setMemoChanged();
            } else
                LibPutMsgDlg(GMSG(MSG_NONXTP));
        }
        else
            LibPutMsgDlg(DMSG(MSG_HASNOTITLE));
        break;
    case 2: /* goto page */
        if ( checktitleok( m_in_tp.txbf ) )  {
            if ( !checkMemoSavedContinue() || memoHasChanged() ) /* continue only with a saved memo */
                break;
            getTitle( m_in_tp.txbf, screen.g_buf); /* misuse screen-copy-buffer for title- and text-buffer */
            /* MW: */
            switch ( loadtext( m_in_tp.txbf, screen.g_buf, &currentfp,
                               &(p_pvmerlin_permanent_data->currentcat) ) ) {
            case TRUE:
              initNewTxtContent();
              break;
            case HALF: { /*ce: almost like new memo, but new Memo gets title+last category  */
                clearTextBuffer();
#ifndef TEST
                p_pvmerlin_permanent_data->currentcat = -1;
#endif
                merlinTxtInit(&m_in_tp, 0);
                enterString(screen.g_buf, FALSE);
                saveParams();
              }
              break;
            default:
              break;
            }
            /* :MW */
        }
        else
            LibPutMsgDlg(DMSG(MSG_HASNOTITLE));
        break;
    }
}



static byte* time_menu = "date&time\xfe" "date\xfe" "time\xfe" "time diff\xff";
int timestamp_first;  /* seconds since last full hour of last 'time'-call */
int timestamp_last;   /* seconds since last full hour of last 'time-diff'-call */
int timestamp_cnt;

void enterTime()
{
    int h,m,s;
    char buf[10];
    getTime( &h, &m, &s );
    sprintf(buf,"%02i:%02i:%02i\r",h,m,s);
    enterString(buf, FALSE);
    /* reset time-diff-info */
    timestamp_first = m*60+s;
    timestamp_last  = timestamp_first;
    timestamp_cnt   = 0;
}

void enterDate()
{
    word y;
    int m,d;
    char buf[12];
    getDate( &y, &m, &d );
    switch ( SYS_DATE_FORMAT ) {
        case 0: /*M/D/Y*/
            sprintf(buf,"%i/%i/%u",d,m,y);
            break;
        case 1: /*D/M/Y*/
            sprintf(buf,"%i.%i.%u",d,m,y);
            break;
        default: /*Y/M/D*/
            sprintf(buf,"%u/%i/%i",y,m,d);
            break;
    }

    enterString(buf, FALSE);
    /* reset time-diff-info */
}

void enterTimeDiff()
{
    int h,m,s;
    int timestamp, d1, dlast, avg;
    char buf[39];
    if ( timestamp_first<0 ) { /* very first call */
        timestamp_first = -1;
        timestamp_last = -1;
        timestamp_cnt = 0;
        LibPutMsgDlg(GMSG(MSG_COUNTER_NOTSTARTED));
        return;
    }
    getTime( &h, &m, &s );
    timestamp = m*60 + s;
    d1 = timestamp - timestamp_first;
    if ( d1<0 ) /* time has passed a full hour */
        d1 += 60*60;
    dlast = timestamp - timestamp_last;
    if ( dlast<0 ) /* time has passed a full hour */
        dlast += 60*60;
    timestamp_cnt++;
    avg = d1/timestamp_cnt;
#ifdef TEST
    switch (PARAM(PARAM_TIMEFORMAT)) {
        case P_TF_NDTA:
#endif
            sprintf(buf,"%i) %i:%02i total=%i:%02i avg=%i:%02i\r",
                (timestamp_cnt>999) ? 999 : timestamp_cnt,
                dlast/60, dlast%60, d1/60, d1%60,
                avg/60, avg%60);
#ifdef TEST
            break;
        case P_TF_NT:
            sprintf(buf,"%i) %i:%02i\r",
                (timestamp_cnt>999) ? 999 : timestamp_cnt,
                d1/60, d1%60 );
            break;
        case P_TF_TD:
            sprintf(buf,"%i:%02i (%i:%02i)",
                d1/60, d1%60, dlast/60, dlast%60 );
            break;
        case P_TF_N:
            sprintf(buf,"%i)",
                timestamp_cnt );
            break;
        case P_TF_T:
            sprintf(buf,"%i:%02i",
                d1/60, d1%60 );
            break;
        case P_TF_D:
            sprintf(buf,"%i:%02i",
                dlast/60, dlast%60 );
            break;
    }
#endif
    enterString(buf, FALSE);
    timestamp_last  = timestamp;
}

/*ce load next/prev page with title title
     always saves current page if changed
     loads next/prev memo if title==null */
/* might be better in loadsave.c, but accesses m_in_tp */
void nextPrevPage( t_searchdir dir, char *title)
{
    int success;
    if ( title && !checktitleok( title ) ) {
        LibPutMsgDlg(DMSG(MSG_HASNOTITLE));
        return;
    }
    if ( currentfp==NEWFILE || p_pvmerlin_permanent_data->currentcat==-1 ) {
        LibPutMsgDlg(DMSG(MSG_STILLNEW));
        return;
    }
    if ( !memoHasChanged()
         || (success = savetext( m_in_tp.txbf, &currentfp, &(p_pvmerlin_permanent_data->currentcat) ) ) == TRUE ) {
        success = loadpage( m_in_tp.txbf, title, dir, &currentfp, &(p_pvmerlin_permanent_data->currentcat) );
        if ( !success ) {
            if ( dir==SEARCH_PREV_PAGE ) {
                LibPutMsgDlg3(GMSG(MSG_LOADINGLAST));
                currentfp = 0xfffe;
            } else {
                LibPutMsgDlg3(GMSG(MSG_LOADING1ST));
                currentfp = 0xffff;
            }
            success = loadpage( m_in_tp.txbf, title, dir, &currentfp, &(p_pvmerlin_permanent_data->currentcat) );
        }
        if ( !success ) { /* should never happen */
            LibPutMsgDlg(GMSG(MSG_ERROR_BUTSAVED));
            clearTextBuffer(); /* here almost like: startNewMemo(); but keeps category */
        }
        /* initNewTxtContentNoSave(); /*ceV3.1 this now also calls saveParams(), but V3.2 disabled it again (too slow) */
        addKeywords(m_in_tp.txbf);
        merlinTxtInit(&m_in_tp, 0);
        setMemoFresh();
    } else if (success==FALSE)
        LibPutMsgDlg(GMSG(MSG_SAVEPAGE_FAILED));
}

static saveCurrentFile()
{
    int saveok = savetext( m_in_tp.txbf, &currentfp, &(p_pvmerlin_permanent_data->currentcat) );
    saveParams(); /* remember currently edited file (title may have changed) + cursor pos */
    if (saveok==TRUE) {
        setMemoFresh();
        LibWinIcnMsg(ICON_OK,341,1); /* Data stored! */
    /* LibPutMsgDlg3("file saved!"); */
    }
    else if (saveok==FALSE)
        LibPutMsgDlg(GMSG(MSG_SAVEPAGE_FAILED));
}

#ifdef TEST2
/*** swap Contacs with Memo mode or back ***/
void swapDataMode()
{
    int tmp;
    byte ptmp;
    /* swap stored titles */
    setPermanentTextParams();
    strncpy( screen.g_buf, p_pvmerlin_permanent_data->fulltitleBak, MAX_TITLE_LEN);  /* misuse screen-copy-buffer for title-swap-buffer */
    strncpy( p_pvmerlin_permanent_data->fulltitleBak, p_pvmerlin_permanent_data->fulltitle, MAX_TITLE_LEN);
    strncpy( p_pvmerlin_permanent_data->fulltitle, screen.g_buf, MAX_TITLE_LEN);
    /* swap other stuff */
    tmp = p_pvmerlin_permanent_data->cursorpos;
    p_pvmerlin_permanent_data->cursorpos = p_pvmerlin_permanent_data->cursorposBak;
    p_pvmerlin_permanent_data->cursorposBak = tmp;
    tmp = p_pvmerlin_permanent_data->currentcat;
    p_pvmerlin_permanent_data->currentcat = p_pvmerlin_permanent_data->currentcatBak;
    p_pvmerlin_permanent_data->currentcatBak = tmp;
    
    ptmp = PARAM(PARAM_KEYWORDS);
    PARAM(PARAM_KEYWORDS) = p_pvmerlin_permanent_data->paramKEYWORDSBak;
    p_pvmerlin_permanent_data->paramKEYWORDSBak = ptmp;
    /*applyParam(PARAM_KEYWORDS); should be called, but not here, because it adds keywords before text buffer is really set! */
    
    ptmp = PARAM(PARAM_LPS);
    PARAM(PARAM_LPS) = p_pvmerlin_permanent_data->paramLPSBak;
    p_pvmerlin_permanent_data->paramLPSBak = ptmp;
    applyParam(PARAM_LPS); 

    p_pvmerlin_permanent_data->currentdatamode ^= 1;
}


/*** file menu under icon ***/
typedef enum {  FM_SWITCH_DM,
                FM_nix,
                FM_LIST_MEMOS,
                FM_LIST_CONTACTS,
                FM_NEW_MEMO,
                FM_NEW_CONTACT,
                /* FM_nix2,
                FM_SAVE */
} t_filemenu;  /* type (enum) not used, only constants */

static byte* file_menu = "Switch data mode\xfe"
                         "---------------------\xfe"
                         "List Memos\xfe"
                         "List Contacts\xfe"
                         "New Memo\xfe"
                         "New Contact\xff";
/*                         "---------------------\xfe"
                         "Save\xff"; */

void fileMenu() {
    byte sel;
    sel = SelWindowExt2(1, 10, NULL, 0, file_menu, NULL );
    if ( sel == 0xff || !checkMemoSavedContinue() )
        return;
    switch (sel) {
        case FM_SWITCH_DM:
            swapDataMode();
            currentfp = 0xffff;
            if ( p_pvmerlin_permanent_data->currentcat != -1       /* last file was not new */
                 && p_pvmerlin_permanent_data->fulltitle[0] != 0   /* title not empty */
                 && loadpage( m_in_tp.txbf, p_pvmerlin_permanent_data->fulltitle,
                              SEARCH_NEXT_PAGE, &currentfp, &(p_pvmerlin_permanent_data->currentcat) ) ) {
                if (p_pvmerlin_permanent_data->cursorpos > strlen( m_in_tp.txbf ) ) /* something has changed in file */
                    p_pvmerlin_permanent_data->cursorpos = 0;
                initNewTxtContent(); 
                m_in_tp.cp = p_pvmerlin_permanent_data->cursorpos;
            }
            else /*'new'*/
                startNewMemo();
            /*restart = TRUE;*/
            break;
        case FM_LIST_MEMOS:
        case FM_LIST_CONTACTS:
          {
            int swapped;
            if ( (sel==FM_LIST_MEMOS && p_pvmerlin_permanent_data->currentdatamode == CONTACTMODE)
                 || (sel==FM_LIST_CONTACTS && p_pvmerlin_permanent_data->currentdatamode == MEMOMODE) ) {
                swapDataMode();
                swapped=1;
            } else
                swapped=0;
            switch ( loadtext( m_in_tp.txbf, NULL, &currentfp,
                               &(p_pvmerlin_permanent_data->currentcat) ) ) {
                case TRUE:
                    initNewTxtContent();
                    break;
                case HALF:
                    startNewMemo();
                    break;
                default: /* swap back after esc */
                    if (swapped)
                        swapDataMode();
                    break;
            }
            break;
          }
        case FM_NEW_MEMO:
        case FM_NEW_CONTACT:
            if ( (sel==FM_NEW_MEMO && p_pvmerlin_permanent_data->currentdatamode == CONTACTMODE)
                 || (sel==FM_NEW_CONTACT && p_pvmerlin_permanent_data->currentdatamode == MEMOMODE) )
                swapDataMode();
            startNewMemo();
            break;
        /*case FM_SAVE:
            saveCurrentFile();
            break;*/
    }
}
#endif

/*** handle title buttons ***/
void titleButtonsPenEvents(TCHSTS *p_tsts, TXTP *p_mintp, int *pch)
{
    int buttonnr = p_tsts->obj - TITLEBUTTON_OBJ_BASE;
    if ( buttonnr < 0 || buttonnr >= NRTITLEBUTTONS )
        return;

    if ( buttonnr == TITLEBUTTON_TITLE ) {
        /* handle pen on title, but dont signal on display */
        if ( p_tsts->act == ACT_BREAK_IN )
            handleMenuBar(pch);
    }
    else if ( LibIconClick( &titleButIcons[buttonnr], p_tsts ) == TRUE )  {
        /* a button was pressed */
        switch ( buttonnr ) {
            case TITLEBUTTON_MODECHANGE:
                /* for future extensions and temporary debug output  */
#ifdef ICONDEBUGCODE
                {
                word m_code;
                word m_sts;
                word m_seg;
                word m_ofs;
                LibGetMode(&m_code, &m_sts, &m_seg, &m_ofs);
                LibPutMsgDlg("PVMode: %x %x %x %x\r"
                             "SCH_(SB)MODE: %x, %x\r"
                             "PASS_MATCH: %x\r"
                             "inSecretMode: %i"
                             ,m_code, m_sts, m_seg, m_ofs,
                             (int)SYS_SCH_MODE, (int)SYS_SCH_SBMODE,
                             (int)SYS_PASS_MATCH,
                             (int)inSecretMode() );
                }
#endif
#ifdef TEST2
                fileMenu();
#else
                merlinhelp( HELPMODE_ABOUT );
#endif
                break;
            case TITLEBUTTON_HELP:
                merlinhelpmenu();
                break;
            case TITLEBUTTON_TOOLS:
                LibFuncPtool();
                break;
            case TITLEBUTTON_CALC:
                LibCalWin(ON);
                break;
            case TITLEBUTTON_TIME:
                switch (SelWindowExt2(titleButTchTbl[TITLEBUTTON_TIME].x1-5, 10, NULL, 0, time_menu, NULL )) {
                    case 0:
                        p_tsts->obj = OBJ_TXTTCH;
                        p_mintp->txtst = TXTDYTIM;
                        setMemoChanged();
                        resetKeywordprefix();
                        break;
                    case 1:
                        enterDate();
                        break;
                    case 2:
                        enterTime();
                        break;
                    case 3:
                        enterTimeDiff();
                        break;
                }
                break;
            case TITLEBUTTON_DEL:
                p_tsts->obj = OBJ_TXTTCH;
                p_mintp->txtst = TXTDEL;  /* does not work; see further down for work around */
                break;
            case TITLEBUTTON_PAGE:
                pageMenu();
                break;
            case TITLEBUTTON_PGUP:
            case TITLEBUTTON_PGDN:
                {
                    t_searchdir dir = ( buttonnr==TITLEBUTTON_PGUP ? SEARCH_PREV_PAGE
                                                                   : SEARCH_NEXT_PAGE);
                    getTitle( m_in_tp.txbf, screen.g_buf ); /* misuse screen-copy-buffer for title-buffer */
                    nextPrevPage( dir, screen.g_buf );
                }
                break;
            case TITLEBUTTON_KEYB:
                keyboardmode = 1;
                restart = TRUE;
                break;
            /* see above case TITLEBUTTON_TITLE:
                handleMenuBar(pch);
                break; */
        }

    }

}

/**** end title buttons ****/


/*MW+ce */
/* Menu-bar processing: */
static void handleMenuBar(int* pch)
{
    word sel;
    sel=merlinmenubar();
    switch (sel) {
    case PDWN_NONE:
        break;
    case PDWN_EDT_CUT:
        /*ch = KEY_NONE;*/
        m_in_tp.txtst = TXTCUT;
        setMemoChanged();
        break;
    case PDWN_EDT_COPY:
        *pch = COPY_STROKE;
        break;
    case PDWN_EDT_PASTE:
        *pch = KEY_NONE;
        m_in_tp.txtst = TXTPASTE;
        setMemoChanged();
        break;
    case PDWN_EDT_NEW:
        if ( !checkMemoSavedContinue() )
            break;
        startNewMemo();
        break;
    case PDWN_EDT_OPEN:
        /* CE: use local copy of cat, so that cat remains unchanged if user cancels loading */
        /* MW: int cat = -1; not needed, done inside loadtext() */
        if ( !checkMemoSavedContinue() )
            break;
        switch ( loadtext( m_in_tp.txbf, NULL, &currentfp,
                           &(p_pvmerlin_permanent_data->currentcat) ) ) {
        case TRUE:
            initNewTxtContent();
            break;
        case HALF:
            startNewMemo();
            break;
        default:
            break;
        }
        break;
    case PDWN_EDT_SAVE:
        saveCurrentFile();
        break;
    case PDWN_EDT_DELETE:
        if ( currentfp == NEWFILE) {
            if ( SelWindowExt2(-1, -1, DMSG(MSG_DELETE_NEW), 0, "yes (same as 'New')\x0FE"
                                                              "no!\x0FF", NULL )
                  == 0) {
                startNewMemo();
            }
        }
        else {
            int sel = SelWindowExt2(-1, -1, DMSG(MSG_DELETE), 0, "yes (completely)\x0FE"
                                                               "yes, but keep text in PVMerlin\x0FE"
                                                               "no!\x0FF", NULL );
            if ( sel == 0 || sel == 1 ) {
                deletefile(currentfp);
                if (sel==0) {
                    if ( SelWindowExt2(-1, 120, GMSG(MSG_DELETE_FOREVER), 0, "yes (undo not possible!)\x0FE"
                                                                      "no!\x0FF", NULL )
                          == 0) {
                        startNewMemo();
                        LibPutMsgDlg3(GMSG(MSG_DELETED));
                    }
                }
                else {  /* we deleted the file, but keep the current text as unsaved */
                    currentfp = NEWFILE;
                    p_pvmerlin_permanent_data->currentcat = -1;
                    p_pvmerlin_permanent_data->cursorpos = 0;
                    setMemoChanged();
                    saveParams(); /* remember that current file is new */
                    LibPutMsgDlg3(GMSG(MSG_DELETED));
                }
            }
        }
        break;

    case PDWN_OPT_SETPARAM:
        /*changeParams();*/
        if (setupParams()==1) /* screen has changed */
            merlinTxtInit(&m_in_tp, m_in_tp.cp);
        saveParams(); /* remember param changes  */
        resetKeywordprefix();
        break;
    case PDWN_OPT_KEYBOARD:
        keyboardmode ^= 1;
        restart = TRUE;
        break;
    case PDWN_OPT_SCRCOPY:
        LibScrShot(&screen);
        break;

    default:
        LibPutMsgDlg("Menu error\rSelected item: %x", sel);
    }
}


/***** lever push stuff *****/

/* !! assume that lever-push-parameter-menu has 1+3 entries!! */

void leverPushMenu(TCHSTS *p_tsts)
{
    byte *strend = screen.g_buf;
    byte *from = param_choices[PARAM_LPS];
#ifdef TEST2
    byte ret[] = { 1, 2, 3, 4, 0, 5, 6, 7, 0xff };
#else
    byte ret[] = { 1, 2, 3, 4, 0, 5, 6, 0xff };
#endif
    int sel;

    /* get lever-push-parameter-menu */
    while ( *from != 0xff )
        *strend++ = *from++;
    /* append some other entries */
    *strend++ = 0xfe;
    strcpy (strend, "-------------------\xfe"
                    "Main menu\xfe"
#ifdef TEST2
                    "Page menu\xfe"
                    "File menu\xff" );
#else
                    "Page menu\xff" );
#endif
    sel = SelWindowExt2(-1, -1, "Action wheel", PARAM(PARAM_LPS), screen.g_buf, ret );

    switch (sel) {
    case 0:
    case 0xff:
        break;
    case 5:
        p_tsts->obj = OBJ_HIC_MBAR; /* processed at a later stage */
        break;
    case 6:
        pageMenu();
        break;
#ifdef TEST2
    case 7:
        fileMenu();
        break;
#endif
    default:
      /*if ( sel <= 4 ) {*/
        sel--; /* now sel matches PARAM_LPS values */
        PARAM(PARAM_LPS) = sel;
        applyParam(PARAM_LPS);
#ifdef TEST
        if (sel == P_LPS_TIME) {
            if (editParamWindow(PARAM_TIMEFORMAT)) /* if user selects 'Time' automatically ask for time format */
                merlinTxtInit(&m_in_tp, m_in_tp.cp);  /* screen has changed */
            applyParam(PARAM_TIMEFORMAT);
        }
#endif
        saveParams();
      /*}*/
    }        
/*ceV32 old code: #else
    if ( sel == 0 ) {
        return;
    }
    else if ( sel <= 4 ) {
        PARAM(PARAM_LPS) = sel-1;
        applyParam(PARAM_LPS);
        saveParams();
    }
    else if ( sel == 5 ) {
        p_tsts->obj = OBJ_HIC_MBAR; /* processed at a later stage *
    }
    else if ( sel == 6 ) {
        pageMenu();
    }
#endif*/
}

#ifdef TEST2
static char paragraphSepChars[] = { '\r', 0xfe }; /* sepchar[MEMOMODE] for Memo , sepchar[CONTACTMODE] for Contacts */
#endif

void handleLPSUpDn(TCHSTS *p_tsts)
{
    byte lpsaction = PARAM(PARAM_LPS);
    char sepchar = 
#ifdef TEST2
        paragraphSepChars[p_pvmerlin_permanent_data->currentdatamode];
#else
        '\r';        
#endif        
    if ( lpsaction == P_LPS_PARAGRAPH) {
        if ( (m_in_tp.cp==0 && p_tsts->obj==OBJ_LPSW_UP) ||
             (m_in_tp.txbf[m_in_tp.cp]==0 && p_tsts->obj==OBJ_LPSW_DOWN) )
            lpsaction = P_LPS_PAGE;
        else if ( p_tsts->obj == OBJ_LPSW_UP ) {
            /* go to 1st char of paragraph, resp. to prev paragraph  */
            do { /* skip empty lines */
                m_in_tp.cp--;
            } while ( m_in_tp.cp>0 && m_in_tp.txbf[m_in_tp.cp]==sepchar );
            /* find 1st char in paragraph */
            while ( m_in_tp.cp>0 && m_in_tp.txbf[m_in_tp.cp-1]!=sepchar )
                m_in_tp.cp--;
            m_in_tp.txtst = TXTREDSP;
        }
        else if ( p_tsts->obj == OBJ_LPSW_DOWN ) {
            /* go to end of paragraph, resp. to next paragraph  */
            /* skip empty lines */
            while ( m_in_tp.txbf[m_in_tp.cp]!=0 && m_in_tp.txbf[m_in_tp.cp]==sepchar )
                m_in_tp.cp++;
            /* find end of paragraph */
            while ( m_in_tp.txbf[m_in_tp.cp]!=0 && m_in_tp.txbf[m_in_tp.cp]!=sepchar )
                m_in_tp.cp++;
            m_in_tp.txtst = TXTREDSP;
        }
    }
    if ( lpsaction == P_LPS_PAGE) {
        t_searchdir dir = ( p_tsts->obj == OBJ_LPSW_UP ? SEARCH_PREV_PAGE
                                                       : SEARCH_NEXT_PAGE);
        getTitle( m_in_tp.txbf, screen.g_buf ); /* misuse screen-copy-buffer for title-buffer */
        nextPrevPage( dir, screen.g_buf );
        if ( PARAM(PARAM_LPS) == P_LPS_PARAGRAPH && p_tsts->obj == OBJ_LPSW_UP ) {
            /*set cursor at the end */
            while ( m_in_tp.txbf[m_in_tp.cp]!=0 )
                m_in_tp.cp++;
            m_in_tp.txtst = TXTREDSP;
        }
    }
    else if ( lpsaction == P_LPS_MEMO ) {
        t_searchdir dir = ( p_tsts->obj == OBJ_LPSW_UP ? SEARCH_PREV_PAGE
                                                       : SEARCH_NEXT_PAGE);
        nextPrevPage( dir, NULL );
    }
    else if ( lpsaction == P_LPS_TIME ) {
        if ( p_tsts->obj==OBJ_LPSW_UP )
            enterTime();
        else /* LPSW_DOWN */
            enterTimeDiff();
    }
}

/**** end lever push ****/


/*ce*/
void exitPVMerlin()
{
    if ( m_in_tp.txbf[0] != 0 && memoHasChanged()) {
        t_dosave dosave;
        if ( PARAM(PARAM_SAVE_ON_EXIT) != P_EXITSAVE_ALWAYS ) {
            dosave = doSaveDialog();
            if ( dosave == DOSAVE_BREAK )
                return;
        }
        else
            dosave = DOSAVE_YES;
        if ( dosave == DOSAVE_YES) {
            int saveok = savetext( m_in_tp.txbf, &currentfp, &(p_pvmerlin_permanent_data->currentcat) );
            if ( saveok != TRUE ) {
                if (saveok==FALSE)
                    LibPutMsgDlg(GMSG(MSG_SAVEPAGE_FAILED));
                return;
            }
            else /* if (saveok==TRUE) */
                setMemoFresh();
        }
    }

    saveParams();
    LibJumpMenu();
}


void translate(int *ch)
{
    if (*ch<8)  /*ce - those values are used for the special copy/past/...-strokes */
        *ch = KEY_NONE;

    switch (*ch) {
    case 0x0D:
        *ch = KEY_RTN;
        break;

    case 0x08:
        *ch = KEY_DEL;
        break;

    case ' ':
        *ch = KEY_SPC;
        break;
    }
}

#ifdef TEST
/************ character ring buffer stuff and chbuf event handling basics ****************/
#define CH_DELAY_BUF_MAX ((unsigned)31)   /* must be (2^x)-1 */

unsigned char ch_delay_buf[CH_DELAY_BUF_MAX+2]; /* ring buffer! (FIFO buffer) */
unsigned ch_buf_first;  /* first char in buffer */
unsigned ch_buf_last;   /* one behind last char -> first==last means empty */

#define CHBUF_FULL (((ch_buf_last+1)&CH_DELAY_BUF_MAX) == ch_buf_first)
#define CHBUF_EMPTY (ch_buf_first == ch_buf_last)
#define CHBUF_LASTCHAR ((int)(ch_delay_buf[(ch_buf_last-1)&CH_DELAY_BUF_MAX]))


#define CH_DELAY ( PARAM(PARAM_CHBUFDELAY)==P_CHBUFDELAY_LONG ? 4 : (2-PARAM(PARAM_CHBUFDELAY)))  /* delay time in 1/2 seconds */
#define CHBUF_DELAY_OVER (ch_buf_delaycount == 0)
#define CHBUF_DELAY_REDUCE (ch_buf_delaycount--)
#define CHBUF_ACTIVE (ch_buf_delaycount<=CH_DELAY)

static TCHTBL far timerTchTbl[2];
static int ch_buf_delaycount;

static int chbufDisplayX;

int chbufAddChar(unsigned char c) /* returns success: FALSE means that buffer was already full */
{
    int xw;
    if (CHBUF_FULL)
        return FALSE;

    xw = LibGetProFontSize( IB_PFONT2, c );
    if (CHBUF_EMPTY || (chbufDisplayX+xw > mainTitleX2) ) { /* init buffer display */
        chbufDisplayX = mainTitleX;
        LibClrBox(mainTitleX,0,mainTitleX2-mainTitleX+1,12);
        LibPutDispBox(mainTitleX,0,mainTitleX2-mainTitleX+1,12);
    }
    LibPutProFont(IB_PFONT2,c,chbufDisplayX,2);
    LibPutDispBox(chbufDisplayX,2,xw,10);
    chbufDisplayX += xw;

    ch_delay_buf[ch_buf_last] = c;  /* enter ch in buffer */
    ch_buf_last = (ch_buf_last+1)&CH_DELAY_BUF_MAX;
    return TRUE;
}


/* unused
int chbufTakeChar()  /* read and remove first character from buffer; returns -1 if emtpy *
{
    int ch;
    if (CHBUF_EMPTY)
       return -1;
    ch = ch_delay_buf[ch_buf_first];
    ch_buf_first = (ch_buf_first+1)&CH_DELAY_BUF_MAX;
    return ch;
}
*/

int chbufPopChar()  /* read and remove last entered character from buffer; returns -1 if emtpy */
{
    int xw;
    if (CHBUF_EMPTY)
        return -1;

    ch_buf_last = (ch_buf_last-1)&CH_DELAY_BUF_MAX; /* pop last */

    xw = LibGetProFontSize( IB_PFONT2, ch_delay_buf[ch_buf_last] );
    chbufDisplayX -= xw;
    if ( chbufDisplayX < mainTitleX ) {
        xw -= mainTitleX - chbufDisplayX;
        chbufDisplayX = mainTitleX;
    }
    LibClrBox(chbufDisplayX,2,xw,10);
    LibPutDispBox(chbufDisplayX,2,xw,10);

    return ch_delay_buf[ch_buf_last]; /* popped char is now directly under last-index */
}

void chbufTchOff()
{
    timerTchTbl[0].act = ACT_NONE;  /* no countdown until first character is entered */
    timerTchTbl[0].obj = OBJ_END;
    ch_buf_delaycount = INT_MAX;
}

void chbufTchOn()
{
    timerTchTbl[0].act = ACT_500MSEC;
    timerTchTbl[0].obj = OBJ_500MSEC;
    ch_buf_delaycount = CH_DELAY;
}

void chbufTchInit()
{
    timerTchTbl[0] = TCHTBL_END; /* is not really the end, .act. and .obj are overridden in call to chbufTchOn() */
    chbufTchOn();
    timerTchTbl[1] = TCHTBL_END;
    LibTchStackPush(timerTchTbl);
    ch_buf_first = 0;
    ch_buf_last = 0;
    ch_buf_delaycount = CH_DELAY;
    ch_delay_buf[CH_DELAY_BUF_MAX+1] = 0;
    chbufDisplayX = 0;
}

void chbufEnterAndFlush() /* enter buffer at cursor and empty buffer */
{
    if ( !CHBUF_EMPTY ) {
        ch_delay_buf[ch_buf_last] = 0;
        enterString(ch_delay_buf+ch_buf_first, TRUE);
        if (ch_buf_first>ch_buf_last) /* buffer content wraps around */
            enterString(ch_delay_buf, TRUE);
        ch_buf_first = 0; /* empty buffer */
        ch_buf_last = 0;
    }
    chbufTchOff();
    LibClrBox(mainTitleX,0,mainTitleX2-mainTitleX+1,12);
    drawProgramName(mainTitleX, TITLE_IS_LONG);
    LibPutDispBox(mainTitleX,0,mainTitleX2-mainTitleX+1,12);
}


#endif


/********** main event handling ****************/

void backspace(TCHSTS *p_tsts, TXTP *p_mintp)
{
    if ( p_mintp->cp > 0) {
        /*select prev. char */
        p_mintp->sr_sp = p_mintp->cp-1;
        p_mintp->sr_ep = p_mintp->cp;
        p_mintp->selrv = TRUE;

        LibTxtInp(KEY_NONE,p_tsts,p_mintp);
        LibTxtDspC(p_mintp);

        LibTxtInp(KEY_DEL,p_tsts,p_mintp);
        myTxtDspC(p_mintp);
        delLastCharFromKeywordPrefix();
        setMemoChanged();
    }
}

/* function is slow, but avoids a re-init of LibTxt... */
/* if first character is \x0cf (left arrow) this performs a backspace (currently not used) */
/* second param says whether the string is passed to keyword-recognition  */
void enterString(char *txt, int notifykeyword)
{
    int l=0;
    if (txt==NULL)
        return;
    if (txt[0]==0xcf) { /* handle backspace at first char */
        backspace(&tsts,&m_in_tp);
        l++;
    }
    while (txt[l]!=0) {
        if ( txt[l] == '\r' ) {
            if ( m_in_tp.rtnen != TRUE) {
                m_in_tp.rtnen = TRUE;
                LibTxtInp(KEY_RTN,&tsts,&m_in_tp);
                m_in_tp.rtnen = FALSE;
                m_in_tp.txtst = TXTREDSP;
                LibTxtInp(KEY_NONE,&tsts,&m_in_tp);
            }
            else
                LibTxtInp( KEY_RTN, &tsts, &m_in_tp);
        }
        else
            LibTxtInp( txt[l], &tsts, &m_in_tp);
        /*LibTxtDspC(&m_in_tp);*/
        l++;
        if (notifykeyword && txt[l]!=0)
            updateKeywordPrefix(m_in_tp.txbf, m_in_tp.cp);
    }
    if (notifykeyword) /* check at the last character */
        checkKeywordPrefix(m_in_tp.txbf, m_in_tp.cp);

    m_in_tp.txtst = TXTREDSP;
    LibTxtInp(KEY_NONE,&tsts,&m_in_tp);
    LibTxtDspC(&m_in_tp);
    setMemoChanged();
    LibPutDisp();
    if (!notifykeyword) /* check at the last character */
        resetKeywordprefix();
}



/*******************************************************************************************/
/*******************************************************************************************/


word no_gdcmt[] = { 0xffff, 0 };  /* no guide-comments in PVM, because LibTxtDsp() is needed
to display the guide comments. however this function always draws a stupid keyboard on the screen */
/*word telGd[] = {
/*    213, /* NAME */
/*    202, /* ADDRESS_(H) */
/*    203, /* FAX_(B) */
/*    204, /* PHONE_(B) */
/*    205, /* E-MAIL */
/*    206, /* EMPLOYER */
/*    207, /* FAX_(H) */
/*    208, /* PHONE_(H) */
/*    209, /* MOBILE */
/*    210, /* NOTE */
/*};*/

void eventLoop(void)
{
    int i;
    int ch;
    int thischarmode;
    bool con = FALSE;

    s_modeButtonsPos mbPos1= { { 159-MODEBUTTON_XSIZE, 159-MODEBUTTON_XSIZE, 159-MODEBUTTON_XSIZE },
                               { 38,  38+MODEBUTTON_YSIZE,  38+2*MODEBUTTON_YSIZE }  };    /*ce Merlin-ModeButtons */
    s_modeButtonsPos mbPos2= { { 67-3*MODEBUTTON_XSIZE, 67-2*MODEBUTTON_XSIZE, 67-MODEBUTTON_XSIZE },
                               { 0,  0,  0 }  };
    s_modeButtonsPos* pmbPos = &mbPos1;

/*    t_pvm_mode pvmmode; */

    initKeyword();

    keyboardmode=0;

    timestamp_first = -1;
    timestamp_last = -1;
    timestamp_cnt = 0;

    /* init permanent data */
    if ( !permanentInit(PVM_PERMANENT_FILENAME, &p_pvmerlin_permanent_data, sizeof(s_pvmerlin_permanent_data) ) ) {
        /* no file -> manual initalization */
        /*p_pvmerlin_permanent_data->mtxbf[0] = 0; /* Initialization of text buffer */
        for (i=0; i<NR_PARAMS; i++)
            PARAM(i)=0;
        clearTextBuffer();
        p_pvmerlin_permanent_data->currentcat = -1;
    }
#ifdef TEST2
    p_pvmerlin_permanent_data->searchMode = 0; /*always start with search switched off */
#endif
    
    /* init screen-shot area */
    screen.x = M_ST_X;
    screen.y = M_ST_Y;
    screen.x_size = 146;
    /* screen.y_size = 126; is set inapplyParam() */
    screen.p_x = 7;
    screen.p_y = 30;

    /*** init merlin-tch */
    merlinTch[0].x1 = M_ST_X+1;
    merlinTch[0].y1 = M_ST_Y+1;
    merlinTch[0].x2 = M_ST_X + ML_WIDTH;
    if (keyboardmode==0)
        merlinTch[0].y2 = M_ST_Y + ML_HEIGHT_FULL;
    else
        merlinTch[0].y2 = M_ST_Y + ML_HEIGHT_SMALL;
    merlinTch[0].act = ML_ACTMASK;
    merlinTch[0].obj = OBJ_MERLIN;
    merlinTch[0].ext = 0x0000;
    merlinTch[1] = TCHTBL_END;


    /*** Text input setting ***/
    /* now part of merlinTxtInit: m_in_tp.st_x = M_ST_X; /* Start coordinate (X) of text display */
    m_in_tp.st_y = M_ST_Y; /* Start coordinate (Y) of text display */
    m_in_tp.ed_x = M_ED_X; /* End coordinate (X) of text display */
    /* set in applyParam: m_in_tp.it_y = M_IT_Y; /* Text display line spacing (Y) */
    /*                    m_in_tp.MAXGYO = M_MAXG; /* Number of text display lines */
    /* m_in_tp.font = IB_PFONT1; /* Display font type */
    m_in_tp.csen = TRUE;   /* Cursor display enabled */
    /* set in applyParam: m_in_tp.rtnen = TRUE; /* CR code display enabled */
    m_in_tp.maxmj = 2048;  /* Maximum number of allowable input characters. */
    /* m_in_tp.txbf = p_pvmerlin_permanent_data->mtxbf; /* Designation of text buffer address */
    m_in_tp.txbf = textbuffer; /* Designation of text buffer address */
    m_in_tp.gdcmt = no_gdcmt;  /* Guidance comment table */
    m_in_tp.txtobj = OBJ_TXTTCH; /* Object when text area is touched. */
    m_in_tp.sbrobj = OBJ_SCR_BAR; /* Object when the scroll bar is touched. */
    m_in_tp.tchtb = InptTch;   /* Pointer for text scroll bar area */
    m_in_tp.s_pos = &m_in_scb; /* Pointer for text and scroll bar information */
    m_in_tp.kw_dbk = NULL;
    m_in_tp.kwb_0 = NULL;
    m_in_tp.kwb_1 = NULL;
    m_in_tp.kwb_2 = NULL;

    /*for (i=0; i<20; i++)
        mtxbf[i]=0xD;
    mtxbf[20] = 0;*/

    for (i=0; i<NR_PARAMS; i++)
        applyParam(i);

    if (inSecretMode()) {
        catsave = p_pvmerlin_permanent_data->currentcat;
#ifdef TEST2
        catbaksave = p_pvmerlin_permanent_data->currentcatBak;
#endif
    }

    /* init text area */
    i =  PARAM(PARAM_START_WITH);
    if ( i == P_SW_LAST && inSecretMode() ) {  /* in secret mode, we do not reload the last file */
        i = P_SW_NEW;
    }
#ifdef TEST2
    /* don't start with a new contact (would mean that PVMerlin starts with a category dialog) */
    if ( p_pvmerlin_permanent_data->currentdatamode == CONTACTMODE
         && ( i == P_SW_NEW
              || ( i==P_SW_LAST && (p_pvmerlin_permanent_data->currentcat == -1
                                    || p_pvmerlin_permanent_data->fulltitle[0] == 0) ) ) ) {
        swapDataMode();
    }
#endif
    if ( i == P_SW_LOAD ) {
        int c = p_pvmerlin_permanent_data->currentcat;
        clearTextBuffer();
        switch( loadtext( m_in_tp.txbf, NULL, &currentfp, &(p_pvmerlin_permanent_data->currentcat)) ) {
        case TRUE:
            saveParams(); /* remember currently edited file-title */
            break;
        case HALF: /* new button*/
            i = P_SW_NEW;
            break;
        case FALSE: /* esc (or load failed) */
            p_pvmerlin_permanent_data->currentcat = c; /*ceV3.1 variables swaped!! was there a bug? */
            i = P_SW_LAST;
            break;
        }
    }
    switch (i) {
    case  P_SW_LAST:
        /* load last memo */
        currentfp = 0xffff;
        /*LibPutMsgDlg3("load: %s %i", p_pvmerlin_permanent_data->fulltitle, p_pvmerlin_permanent_data->currentcat );*/
        if ( p_pvmerlin_permanent_data->currentcat != -1       /* last file was not new */
             && p_pvmerlin_permanent_data->fulltitle[0] != 0   /* title not empty */
             && loadpage( m_in_tp.txbf, p_pvmerlin_permanent_data->fulltitle,
                          SEARCH_NEXT_PAGE, &currentfp, &(p_pvmerlin_permanent_data->currentcat) ) ) {
            if (p_pvmerlin_permanent_data->cursorpos > strlen( m_in_tp.txbf ) ) /* something has changed in file */
                p_pvmerlin_permanent_data->cursorpos = 0;
            setMemoFresh();
            break;
        }
        /* else 'new' */
    case P_SW_NEW:
        startNewMemo();
        break;
    }
    addKeywords(m_in_tp.txbf);
    m_in_tp.cp = p_pvmerlin_permanent_data->cursorpos;

    while (1)
    {
        restart = FALSE;
        /* pvmmode = PVM_IDLE_MODE; */
        if (keyboardmode==0)
            merlinTch[0].y2 = M_ST_Y + ML_HEIGHT_FULL;
        else
            merlinTch[0].y2 = M_ST_Y + ML_HEIGHT_SMALL;
        mlInit(merlinTch, TRUE, OBJ_TXTTCH, TRUE);
        applyParam(PARAM_LINESPACING);

        drawBackground();
        merlinTxtInit(&m_in_tp, m_in_tp.cp); /* Initialization of text input */
        /*** Text touch area setting ***/
        LibTchStackClr();
        LibTchStackPush(NULL);
        LibTchStackPush(TchHardIcon);
#ifdef TEST
        chbufTchInit();
#endif
        LibTxtTchSet(&m_in_tp); /* Text touch area PUSH */
#ifdef TEST
        chbufTchOff();
#endif

        /*LibTchStackPop();*/
        LibTchStackPush(merlinTch);
        LibTchStackPush(TchActionKey); /*actionWheelTch);*/

        /*ce Merlin-ModeButtons and titleButtons */
        if (keyboardmode==0)
            pmbPos = &mbPos1;
        else
            pmbPos = &mbPos2;
        titleButtonsInit();
        if ( MERLINACTIVE() )
            modeButtonsInit( pmbPos );
        LibTchInit();
        if (keyboardmode==0) {
            /*ce some cosmetics for the modeButtons and pgup/pgdn */
            /*LibGdsLineClr( pmbPos->x[0], 13, 159, 13 );*/
            /* LibGdsLine( mbPos.x[0], mbPos.y[0]-1, 159, mbPos.y[0]-1 ); */
            LibGdsLine( pmbPos->x[0]-1, 13, pmbPos->x[0]-1 , pmbPos->y[NRMODEBUTTONS-1] + MODEBUTTON_YSIZE -1 );
            /*LibGdsLineMesh( mbPos.x[0]-1, mbPos.y[NRMODEBUTTONS-1]+MODEBUTTON_YSIZE, 159, mbPos.y[NRMODEBUTTONS-1] + MODEBUTTON_YSIZE ); */
            LibGdsLineClr( 159, pmbPos->y[0]-1, 159 , 159 ); /* mbPos.y[NRMODEBUTTONS-1] + MODEBUTTON_YSIZE -1 ); */
        }
        LibPutDisp();

        myTxtDspC(&m_in_tp);

        /*** Data input loop ***/
        do {
            if ( /*ce4 deleted*/ getEventMode() == PVM_IDLE_MODE && (/*m_in_tp.sr_sp == m_in_tp.sr_ep ||*/ 
                 !m_in_tp.selrv) ) {
                LibCurBlnkOn(m_in_tp.c_xp, m_in_tp.c_yp, 2, m_in_tp.it_y);
                con = TRUE;
            } else {
                LibCurBlnkOff();
                con = FALSE;
            }

            if (keywordsAvailable())
                openKeywordWindow( m_in_tp.c_xp, min(m_in_tp.c_yp,81) ); /* avoid window to be on lower half of display */

            /*** wait for event **/
            if (keyboardmode==0) {
                LibTchWait(&tsts);
                ch = -1;
            } else {
                ch = LibGetKeyM(&tsts);
                if (ch==KEY_NEXT) {
                    keyboardmode = 0;
                    restart = TRUE;
                }
            }

            /*LibPutMsgDlg2("tch:%i,%i\r%x %x %x\r%x %x %x %x",
                           tsts.x,tsts.y,
                           tsts.obj, tsts.act, tsts.ext,
                           (unsigned)(tsts.istr[0]), (unsigned)(tsts.istr[1]), (unsigned)(tsts.istr[2]), (unsigned)(tsts.istr[3]) );*/

            thischarmode = getCharMode();

            /*** handle keywords and action wheel ***/
            if (keywordsAvailable()) {
                closeKeywordWindow();
#ifdef TEST
                if ( !CHBUF_EMPTY )   /* buf not empty, but something happend */
                     chbufEnterAndFlush();
#endif
                /* lever push switch -  in select-keyword-mode*/
                if ( tsts.obj == OBJ_LPSW_PUSH ) {
                    m_in_tp.txtst = TXTREDSP; /* just get rid of keyword-display */
                }
                else if ( tsts.obj == OBJ_LPSW_UP || tsts.obj == OBJ_LPSW_DOWN ) {
                    enterString( getKeywordRemainderAt( tsts.obj==OBJ_LPSW_UP ? 0 : 1 ), FALSE );
                }
            }
            else {
                /* lever push switch -  normal */
                if ( tsts.obj == OBJ_LPSW_PUSH ) {
                    if (con)
                        LibCurBlnkOff();
#ifdef TEST
                    if ( !CHBUF_EMPTY )  /* buf not empty, but something happend */
                        chbufEnterAndFlush();
#endif
                    leverPushMenu(&tsts);  /* otherwise call menu */
                    if (con)
                        LibCurBlnkOn(m_in_tp.c_xp, m_in_tp.c_yp, 2, m_in_tp.it_y);
                }
                else if ( tsts.obj == OBJ_LPSW_UP || tsts.obj == OBJ_LPSW_DOWN ) {
                    if (con)
                        LibCurBlnkOff();
#ifdef TEST
                    if ( !CHBUF_EMPTY )   /* buf not empty, but something happend */
                        chbufEnterAndFlush();
#endif
                    handleLPSUpDn(&tsts);
                    if (con)
                        LibCurBlnkOn(m_in_tp.c_xp, m_in_tp.c_yp, 2, m_in_tp.it_y);
                }
            }

#ifdef TEST
            if ( tsts.obj == OBJ_500MSEC && getEventMode() == PVM_IDLE_MODE ) {  /* delayed entering of characters */
                /*LibGdsReverse( 0, 0, 6, 6 ); LibPutDispBox( 0, 0, 6, 6 );*/
                if ( CHBUF_DELAY_OVER ) {
                    /*LibGdsReverse( 7, 7, 12, 12 ); LibPutDispBox( 7, 7, 12, 12 );*/
                    chbufEnterAndFlush();     /*option 2: enter whole buffer (better)
                                                    and stop countdown until next character is entered */
                }
                else /*if (ch_buf_delaycount <= CH_DELAY)*/
                    CHBUF_DELAY_REDUCE;
                continue;
            } else
#endif
            /*** handle pen-input (state machine for pen-input-mode) ***/
            if ( tsts.obj == OBJ_MERLIN || tsts.obj == OBJ_TXTTCH ) {  /*ce all new */
                if ( !MERLINACTIVE() )
                    tsts.obj = OBJ_TXTTCH;
                else {
                    int mlRet;
                    tsts.obj = OBJ_MERLIN;
                    mlRet = mlHandleOrPassEvent(&tsts);
                    switch (mlRet) {
                        case MERLIN_PENSTROKE:
                          /*ceV3.1 faster loop cut-off to get next event */
                           continue;
                        case MERLIN_PASSTHROUGH_START:
                            LibGdsReverse( pmbPos->x[0], pmbPos->y[0], pmbPos->x[NRMODEBUTTONS-1]+MODEBUTTON_XSIZE-1, pmbPos->y[NRMODEBUTTONS-1]+MODEBUTTON_YSIZE-1 ); /* turn indicator on */
                            LibPutDispBox( pmbPos->x[0], pmbPos->y[0], pmbPos->x[NRMODEBUTTONS-1]+MODEBUTTON_XSIZE-pmbPos->x[0], pmbPos->y[NRMODEBUTTONS-1]+MODEBUTTON_YSIZE-pmbPos->y[0] );
                            break;
                        case MERLIN_PASSTHROUGH_END:
                            /* end selection or cursor positioning */
                            LibGdsReverse( pmbPos->x[0], pmbPos->y[0], pmbPos->x[NRMODEBUTTONS-1]+MODEBUTTON_XSIZE-1, pmbPos->y[NRMODEBUTTONS-1]+MODEBUTTON_YSIZE-1 ); /* turn indicator on */
                            LibPutDispBox( pmbPos->x[0], pmbPos->y[0], pmbPos->x[NRMODEBUTTONS-1]+MODEBUTTON_XSIZE-pmbPos->x[0], pmbPos->y[NRMODEBUTTONS-1]+MODEBUTTON_YSIZE-pmbPos->y[0] );
                            resetKeywordprefix();
                            if ( m_in_tp.selrv != TRUE) {
                                /* if cursor was set (not a full selection) we fall out of extended mode */
                                t_char_mode cm = getCharMode();
                                if ( cm == ALPHA_EXT_MODE )
                                    setCharMode( ALPHA_MODE );
                                else if ( cm == NUMERAL_EXT_MODE )
                                    setCharMode( NUMERAL_MODE );
                                modeButtonsDisplayMode();
                            }
                        case MERLIN_PASSTHROUGH:
                            break;
                        case MERLIN_IGNORE: /* maybe 'continue' would be right here, too */
                            break;
                        default: /* character recognized */
                            ch = mlRet;
 #ifdef TEST
                            if ( ch != 0
                                 && (PARAM(PARAM_CHBUF) == P_CHBUF_ON
                                     || ( PARAM(PARAM_CHBUF) == P_CHBUF_200 && m_in_tp.mjcnt>200 )
                                     || ( PARAM(PARAM_CHBUF) == P_CHBUF_500 && m_in_tp.mjcnt>500 ) ) ) {
                                if ( CHBUF_FULL                         /* max buf-capacity reached */
                                     || (ch <0x21 && (ch!=0x08 || CHBUF_EMPTY)) || ch > 0xEF  /* special character input ends buffering */
                                     || m_in_tp.selrv == TRUE           /* do not use buffer if something is selected */
                                     || (CHBUF_EMPTY && strchr(accent, ch)/*ch is accent*/) ) {  /* do not use buffer if first char is accent */
                                    if (CHBUF_ACTIVE)
                                        chbufEnterAndFlush();
                                } else { /* put new char in buffer */
                                    if (ch == 0x08) { /* handle backspace within buffer */
                                        if ( !CHBUF_EMPTY )
                                           chbufPopChar();  /* else leave backspace in ch */
                                    } else {
                                        int newch;
                                        /* handle accents within buffer */
                                        if ( !CHBUF_EMPTY && thischarmode==ALPHA_MODE
                                             && (newch = checkAccentable( CHBUF_LASTCHAR, ch )) ) {
                                            chbufPopChar();   /* delete prev. char in buf */
                                            ch = newch;
                                        }
                                        chbufAddChar(ch);
                                    }
                                    ch = -1;
                                    chbufTchOn();
                                }
                            } else if (!CHBUF_EMPTY) /* ch==0 mode changed with a stroke during write to buffer, so reset delay counter as if a character was entered */
                                chbufTchOn();
                #endif
                    }
                }
            }
            else {
                putDispAtKeywordWindow(); /* make keyword window really go away from screen */
#ifdef TEST
                if ( !CHBUF_EMPTY && ch != 0 )   /* buf not empty, but something happend that is not a wiring mode change */
                     chbufEnterAndFlush();
#endif
                /*** handle a selected keyword ***/
                if (tsts.obj == OBJ_KEYWORD ) {
                    /* enter selected word */
                    putDispAtKeywordWindow(); /* make keyword window really go away from screen */
                    enterString( getKeywordRemainder(&tsts), FALSE );
                }

                /*** handle mode buttons ***/
                else if (tsts.obj >= MODEBUTTON_OBJ_BASE  &&  tsts.obj < MODEBUTTON_OBJ_BASE+NRMODEBUTTONS ) {
                    modeButtonsPenEvent(&tsts);
                }

                /*** handle  title buttons ***/
                else if (tsts.obj >= TITLEBUTTON_OBJ_BASE  &&  tsts.obj < TITLEBUTTON_OBJ_BASE+NRTITLEBUTTONS ) {
                    if (con)
                        LibCurBlnkOff();
                    titleButtonsPenEvents( &tsts, &m_in_tp, &ch );
                    if (m_in_tp.txtst == TXTDEL) {
                        ch = KEY_DEL;
                        m_in_tp.txtst = TXTNOP;
                    }
                    if (con)
                        LibCurBlnkOn(m_in_tp.c_xp, m_in_tp.c_yp, 2, m_in_tp.it_y);
                }

                /*** handle esc ***/
                else if (tsts.obj == OBJ_HIC_ESC ) {
                    if ( keyboardmode ) {
                        keyboardmode = 0;
                        restart = TRUE;
                    }
                    else
                        exitPVMerlin();
                }

                /*** handle menu bar (MW+ce) ***/
                else if (tsts.obj == OBJ_HIC_MBAR) {
                    if (con)
                        LibCurBlnkOff();
                    handleMenuBar(&ch);
                    if (con)
                        LibCurBlnkOn(m_in_tp.c_xp, m_in_tp.c_yp, 2, m_in_tp.it_y);
                }

                /*** other hard icons -- does not work ***
                else if ( tsts.obj == OBJ_MYHARDICON )
                       *(tsts.obj >= OBJ_HIC_POS1 && tsts.obj <= OBJ_HIC_POS5 )  { *
                    p_pvmerlin_permanent_data->cursorpos = m_in_tp.cp;
                    saveParams();
                        LibTchStackPush(TchHardIcon);
                    LibTchHardIcon(&tsts,0);
                }
                */
            }

            /*** handle character-inputs ***/
            /* some come indiretly from a menu-selection etc. so here starts a new if-branch */
            if ( ch!=KEY_NONE && ch!=-1 )
            {
                if (ch==0x08) { /* BackSpace */
                    if (  m_in_tp.selrv != TRUE ) {
                        backspace(&tsts,&m_in_tp);
                        ch = KEY_NONE;
                    }
                }
                else if ( ch==COPY_STROKE ) {
                    resetKeywordprefix();
                    if (  m_in_tp.selrv == TRUE ) {
                        m_in_tp.txtst = TXTCOPY;
                    } else {
                        int cp = m_in_tp.cp;
                        /* nothing selected: temporarily select whole text area */
                        m_in_tp.sr_sp = 0;
                        m_in_tp.sr_ep = strlen(m_in_tp.txbf);
                        m_in_tp.selrv = TRUE;
                        LibTxtInp(KEY_NONE,&tsts,&m_in_tp);
                        myTxtDspC(&m_in_tp);
                        /* copy */
                        m_in_tp.txtst = TXTCOPY;
                        LibTxtInp(KEY_NONE,&tsts,&m_in_tp);
                        myTxtDspC(&m_in_tp);
                        /* deselect */
                        m_in_tp.cp = cp;
                        m_in_tp.sr_sp = 0;
                        m_in_tp.sr_ep = 0;
                        m_in_tp.selrv = FALSE;
                        m_in_tp.txtst = TXTREDSP;
                        LibTxtInp(KEY_NONE,&tsts,&m_in_tp); /*ce dirty, but does not work otherwise */
                        myTxtDspC(&m_in_tp);
                        /*backspace(&tsts,&m_in_tp);*/
                    }
                }
                else if ( ch==PASTE_STROKE ) {
                    m_in_tp.txtst = TXTPASTE;
                    setMemoChanged();
                    resetKeywordprefix();
                }

                translate(&ch);

                /* test for accents/umlaut-strokes */
                if ( ch!=KEY_NONE && m_in_tp.cp > 0 && thischarmode==ALPHA_MODE && m_in_tp.selrv != TRUE ) {
                    int newch = checkAccentable( m_in_tp.txbf[m_in_tp.cp-1], ch );
                    if ( newch != 0 ) {
                        /* MW: corrected a buglet: previously when the character was accentable but didn't match the
                           current accent the accent was deleted. Now it is preserved, just as for non-accentable chars */
                        backspace(&tsts,&m_in_tp);   /* delete prev. char */
                        ch = newch;
                    }
                }

                /* auto-copy */
                else if ( m_in_tp.selrv == TRUE && ch!=KEY_DEL && ch!=KEY_NONE
                          && abs(m_in_tp.sr_ep - m_in_tp.sr_sp) > 3 ) {
                    /* save internal txt-state */
                    word sp     = m_in_tp.sr_sp;
                    word ep     = m_in_tp.sr_ep;
                    int  xp     = m_in_tp.sr_xp;
                    int  yp     = m_in_tp.sr_yp;
                    byte ipdpst = m_in_tp.ipdpst;
                    byte txtst  = m_in_tp.txtst;
                    word srln   = m_in_tp.srln;
                    /* copy */
                    LibPutMsgDlg3(GMSG(MSG_AUTOCOPY));
                    m_in_tp.txtst = TXTCOPY;
                    LibTxtInp(KEY_NONE,&tsts,&m_in_tp);
                    LibTxtDspC(&m_in_tp);
                    /* reselect text */
                    m_in_tp.sr_sp = sp;
                    m_in_tp.sr_ep = ep;
                    m_in_tp.sr_xp = xp;
                    m_in_tp.sr_yp = yp;
                    m_in_tp.ipdpst= ipdpst;
                    m_in_tp.txtst = txtst;
                    m_in_tp.srln = srln;
                    m_in_tp.selrv = TRUE;
                    /* character ch will now overrite the selection as usual */
                }

                /*workaound: allow entering a return-character although
                             LibTxt... disallows it in 'rtnen==FALSE'-mode */
                if ( ch==KEY_RTN && m_in_tp.rtnen != TRUE) {
                    m_in_tp.rtnen = TRUE;
                    LibTxtInp(ch,&tsts,&m_in_tp);
                    checkKeywordPrefix(m_in_tp.txbf, m_in_tp.cp);
                    m_in_tp.rtnen = FALSE;
                    m_in_tp.txtst = TXTREDSP;
                    setMemoChanged();
                    ch = KEY_NONE;
                }

#ifdef ALLOWDEBUGCODE
                if ( PARAM(PARAM_DEBUG) == P_DBG_FULL ) {
                    LibPutMsgDlg3("m: %i", getEventMode() );
                }
#endif

                if (ch!=KEY_NONE && ch!=KEY_SHIFT && ch!=KEY_NEXT && ch!=KEY_CHANGE && ch!=KEY_SMBL && ch!=KEY_CODE && ch!=KEY_CAPS )
                    setMemoChanged();
            } /* end special ch-treatments */
            else
                ch = KEY_NONE;  /* instead of translate(&ch) */

            LibTxtInp(ch,&tsts,&m_in_tp);
            if (ch!=KEY_NONE)
                checkKeywordPrefix(m_in_tp.txbf, m_in_tp.cp);
            myTxtDspC(&m_in_tp); /* Display during text input - ceV3.1 moved down from top of loop*/

        } while (!restart);

        if (con)
            LibCurBlnkOff();

    }

}


/* ---- parameter menues ---- */

/*ce show current Parameter settings and let user select one to edit*/
int selParamWindow()
{
    byte *strend = param_menu;
    byte *p_param_value;
    int i,v;

    for (i=0;i<NR_PARAMS;i++) {
        /* add param-descr as menu entry */
        p_param_value = param_names[i];
        while ( *p_param_value!=0 && *p_param_value!=0xfe ) {
            *strend = *p_param_value;
            p_param_value++;
            strend++;
        }
        *strend=':';
        strend++;
        *strend=' ';
        strend++;
        /* append current value */
        p_param_value = param_choices[i];
        v = 0;
        while ( v < PARAM(i) && *p_param_value!=0 ) { /* find string for current value */
            while ( *p_param_value!=0xfe && *p_param_value!=0xff && *p_param_value!=0 )
                p_param_value++;
            v++;
            p_param_value++;
        }
        while ( *p_param_value!=0xfe && *p_param_value!=0xff  && *p_param_value!=0 ) {  /* append it */
            *strend = *p_param_value;
            p_param_value++;
            strend++;
        }
        *strend = 0xfe;
        strend++;
    }
    strend--;
    *strend = 0xff;
    strend++;
    *strend = 0;

    return SelWindowExt2(-1, 14, "PVMerlin parameters", 0, param_menu, NULL );
}

/*ce edit one parameter
returns 0 if no change occured,
        1 if parameter value changed */
int editParamWindow(int p)
{
    int prev_paramvalue = PARAM(p);
    byte *p_param_value = param_choices[p];
    int result;

    while ( *p_param_value!=0 ) { /* count entries */
        while ( *p_param_value!=0xfe && *p_param_value!=0xff )
            p_param_value++;
        p_param_value++;
    }

    result = SelWindowExt2(50, 10+9*p, param_names[p], PARAM(p), param_choices[p], NULL );
    if ( result != 255 ) {
        PARAM(p) = result;
        if ( prev_paramvalue != PARAM(p) )
            return 1;
    }
    return 0;
}

int setupParams()
{
    int choice,screen_changed=0;
    while ( (choice=selParamWindow()) != 255) { /* if anything else but <esc> has been pressed */
        /* set param */
#ifndef TEST
        if (choice==PARAM_FONT) { /* special handling of font-selection */
            /*!! assumes that IB_PFONT1==0 and IB_PFONT2==1 */
            if (LibSelectFont(&PARAM(PARAM_FONT)))
                screen_changed = screen_changed | applyParam(choice);
        } else 
#endif
        if (editParamWindow(choice)) {
            screen_changed = screen_changed | applyParam(choice);
        }
    }
    return screen_changed;
}

/***/

bool memoHasChanged() {
    return memo_changed;
}

static void drawFileState()
{
    int x = ( WIDEICON ? 13 : 9 );  /* titleButTchTbl[TITLEBUTTON_PAGE].x1-3; */

    LibClrBox(  x, 1, 2, 10);

    if (memoHasChanged()) {
        if ( currentfp == NEWFILE ) {
            LibGdsLineMesh( x,  2, x,  6 );
            LibGdsLineMesh(x+1, 1,x+1, 7 );
            LibGdsLine( x,  9, x, 10 );
            LibGdsLine(x+1,10,x+1,10 );
        }
        else {
            LibGdsLine( x,  1, x,  6 );
            LibGdsLine(x+1, 1,x+1, 7 );
            LibGdsLine( x,  9, x, 10 );
            LibGdsLine(x+1, 9,x+1,10 );
        }
    }

    LibPutDispBox(  x, 1, 2, 10);
}

void setMemoChanged()
{
    if (memo_changed==TRUE)
        return;
    memo_changed=TRUE;
    drawFileState();
}

static void setMemoFresh()
{
    if (memo_changed==FALSE)
        return;
    memo_changed=FALSE;
    drawFileState();
}


/***/

#if 0
static void penEvent( const TCHSTS *tsts)
{
        Point p;
        bool draw = TRUE;
        int i;

        p.x = tsts->x; p.y = tsts->y;
        switch (tsts->act) {
        case ACT_MAKE:
                if (current)
                        deletePattern(current);
                mlPaint();
                current = newPattern(100,'a');
                addPoint(current, p);
                x = p.x; y = p.y;
                break;

        case ACT_MOVE:
        case ACT_BREAK_IN:
                addPoint(current, p);
                break;

        case ACT_MOVE_OUT:
                addPoint(current, p);
                LibTchInit();
                break;

        default:
                draw = FALSE;
        }
        if (draw) {
                i = current->nPoints - 1;
                LibGdsLine(x,y,current->pv[i].x,current->pv[i].y);
                LibPutDisp();
                x = p.x; y = p.y;
        }
}

static void print(char *str)
{
        int len = strlen(str);
        int i = 0;

        while (i < 3 && LibSrlSendBlock(str, len) != IW_SRL_NOERR) {
                LibWait(IB_125MWAIT);
                i++;
        }

}

static void send()
{
        char buf[20];
        int i;
        SRL_STAT sr;

        sr.port = IB_SRL_COM2;
        sr.speed = IB_SRL_38400BPS;
        sr.parit = IX_SRL_NONE;
        sr.datab = IX_SRL_8DATA;
        sr.stopb = IX_SRL_1STOP;
        sr.fctrl = IX_SRL_RSCS;
        LibSrlPortOpen(&sr);

        sprintf(buf,"S %d %d\r\n",curch,current->nPoints);
        print(buf);
        for (i=0; i < current->nPoints; i++) {
                sprintf(buf,"P %d %d\r\n",
                        current->pv[i].x - JotPad[0].x1,
                        current->pv[i].y - JotPad[0].y1);
                print(buf);
        }

        LibSrlPortClose();
}
#endif

int main()
{
        initApp();
        /*drawBackground();
        LibPutDisp();*/

        /*ce moved
        LibTchStackClr();
        LibTchStackPush(NULL);
        LibTchStackPush(TchHardIcon);
        LibTchStackPush(JotPad);
        LibTchInit();
        */

        eventLoop();
}
