/*********************************************************
 * ml_help.c
 *
 * Copyright (c) Christian Ensel
 *
 * Help for pv-merlin-modes and strokes 
 *********************************************************/

/*ce !! indexes of most special paterns (for COPY, PASTE, accents, ...) are hard coded in the following defines and also in some
   draw...pattern-calls in merlinhelp(), these may have to be changed if there index in models.h changes */

extern int npatterns[];
#define UP_STROKE_INDEX (npatterns[ALPHA]-9)  /* count special characters from the back. this is more robust to changes in the letter-models */
#define NUM_STROKE_INDEX (npatterns[ALPHA]-8)
#define COPY_STROKE_INDEX (npatterns[ALPHA]-2)
#define PASTE_STROKE_INDEX (npatterns[ALPHA]-1)

#include <stdrom.h>
#include "define.h"
#include "libc.h"
#include "md.h"

#include "util.h"
#include "mod_symb.h"
#include "ml_help.h"
#include "wizard.h"
#include "pattern.h"

extern Pattern FAR *models[];

void merlinhelpmenu()
{
    int result;
    byte cmnt[]="General Help\xfe"
                "Writing Modes\xfe"
                "Letters\x0fe"
                "Numeric Mode\x0fe"
                "Extended Characters\x0fe"
                "Extended Numeric Symbols\x0fe"
                "Accents and Umlauts\x0fe"
                "Memos and Pages\x0fe"
                "About\xff";
    byte rtv[10+1] = { HELPMODE_GENERAL, HELPMODE_MODES, HELPMODE_ALPHA, HELPMODE_NUM, 
                       HELPMODE_EXT, HELPMODE_EXT_NUM, HELPMODE_ACC, HELPMODE_PAGES, HELPMODE_ABOUT, 0xff};
/*    SLW2_ST slw;
    slw.x=9;
    slw.y=25;
    slw.xs=138;
    slw.ity=9;
    slw.np=0;
    slw.t_xs=3;
    slw.rtv=rtv;
    slw.cmnt=cmnt;
*/
    while ( (result=SelWindowExt2(-1, -1, "PVMerlin - Help", 0, cmnt, rtv )) != 255) /* if anything else but <esc> has been pressed */
        merlinhelp(result);
}               

#define WINX 3
#define WINY 3


/* draw a pattern */
void drawOnePattern(int x, int y, int model, int nr )
{
    int i,downshift=0;
    Pattern FAR *pat = &models[model][nr];
    
    if ( pat->ascii == 0x08 || pat->ascii == 0x20 || pat->ascii == 0x14
         || pat->ascii == '_' || pat->ascii == '-' || pat->ascii == '=' )
         downshift = 11;
    for (i = 0; i < pat->nPoints-1; i++) {
        LibGdsLine( WINX + x + (pat->pv[i].x >>2)  , WINY + y+ (pat->pv[i].y >>2) + downshift , 
                    WINX + x + (pat->pv[i+1].x >>2), WINY + y+ (pat->pv[i+1].y >>2) + downshift);
    }
    LibBox( x + (pat->pv[0].x>>2)-1 , y + (pat->pv[0].y>>2)-1 + downshift, 3, 3, 1);
    LibDotOn( x + (pat->pv[0].x>>2) , y + (pat->pv[0].y>>2) + downshift);
    
    /*if ( pat->ascii == UP_STROKE )
        LibPutProStr(IB_PFONT2, x, y+19, "CAP", 159 );
    else if ( pat->ascii == NUM_STROKE )
        LibPutProStr(IB_PFONT2, x, y+19, "NUM", 159 );
    else */ if ( pat->ascii == COPY_STROKE )
        LibPutProStr(IB_PFONT1, WINX + x, WINY + y+27, "Copy", 159 );
    else if ( pat->ascii == PASTE_STROKE )
        LibPutProStr(IB_PFONT1, WINX + x, WINY + y+27, "Paste", 159 );
    else if ( pat->ascii == 0x08 )
        LibPutProStr(IB_PFONT1, WINX + x, WINY + y+24,  "<-BS", 159 );
    else if ( pat->ascii == 0x20 )
        LibPutProStr(IB_PFONT1, WINX + x, WINY + y+24,  "Space", 159 );
    else if ( pat->ascii == 0x0D )
        LibPutProStr(IB_PFONT1, WINX + x, WINY + y+24, "Enter", 159 );
    else if ( (pat->ascii >= 0x11 && pat->ascii <= 0x14) || pat->ascii=='"' )
        LibPutProFont(IB_PFONT2, pat->ascii, WINX + x, WINY + y+27 );
    else if ( pat->ascii > 0x2 )
        LibPutProFont(IB_PFONT2, pat->ascii, WINX + x, WINY + y+24 );
}

void drawPatterns(int xs, int ys, int model, int nr1, int nre )
{
    int n,i;
    for ( n=nr1, i=0; n<=nre; n++, i++) {
        drawOnePattern( xs + (i%5)*31, ys + (i/5)*36, model,n);
    }
}


#define OBJ_FULLSCREEN 0x8FFE
static TCHTBL far fullscreen_tchtbl[] = {
       { 0,0, 159, 159, ACT_BREAK_IN, OBJ_FULLSCREEN, 0x000 },
       { 0,0,0,0,ACT_NONE,OBJ_END,0x0000 }
};

/* returns TRUE for normal touch, FALSE for <esc> */
int waitfornextpage()
{
    int res=TRUE;
    res = waitForClick();
    LibClrBox(1,1,152,152);
    LibPutDisp();
    return res;
}

int printHelpLines(int x, int y, char **txt) 
{
    return printCharArray(x,y,txt,152,IB_PFONT1);
}


char* helptxt_general_p1[] = {
    BOLD "General information:",
    YSPACE3,
    "PVMerlin uses no separate",
    "input field to enter text.",
    "Simply draw characters on the",
    "text area. They will be inser-",
    "ted at the cursor position.",
    "Hold the pen down for a short",
    "moment to place the cursor or",
    "to select text.",
    YSPACE3,
#ifdef TEST2 /* save more space */
/*    "You find images of the pen",
    "strokes in the help menu.",*/
#else
    "Please remember that PVMerlin",
    "does not perform real hand-",
    "writing recognition, but your",
    "strokes have to follow certain",
    "pre-set patterns. However,these",
    "patterns look mainly like normal",
    "lower-case letters and therefore",
    "are easy to write.",
#endif    
    ""
};

char* helptxt_general_p2[] = {
    BOLD "Hints:",
    YSPACE3,
    "When writing characters, do not",
    "hold the pen down before you",
    "start to draw strokes. If you", 
    "wait too long, you will start", 
    "to select text instead.",
    "Better move pen immediately", 
    "after (almost while) you start", 
    "to touch the display. Pay extra",
    "attention on the start and end-",
    "points of the strokes.", 
    YSPACE3,
#ifdef TEST2 /* save more space */
    "A '!' at the title bar shows that", 
    "the current text is modified."
#else
    "If the current text is modified",
    "and still unsaved, you will see",
    "a '!' between the icon and the", 
    "PVMerlin title. Pressing <Esc>", 
    "will then not exit immediately", 
    "but ask whether you want to",
    "save first.", 
#endif
    ""
};

char* helptxt_modes_p1[] = {
    BOLD "Writing Modes:",
    YSPACE3,
    "In normal writing mode you can", 
#ifdef TEST2 /* save more space */
    "write any lowercase letter or", 
    "change input mode by writing",
#else
    "write any lowercase letter and", 
    "some special strokes for 'space'",
    "'new line', 'Copy', 'Paste', etc."
    "There are 4 more writing modes",
    "for 'capital letters', 'numbers'",
    "and two sets of extended",
    "characters.", 
    YSPACE3,
    "Modes are changed by writing",
#endif
    "certain simple strokes or by",
    "clicking one or more times",
    "on one of the three mode indi-",
    "cators above the scroll-bar.", 
    ""
};

char* helptxt_modes_p2a[] = {
    "Draw a simple upward-",   
    "stroke to write a single",
    "capital letter.",
    "Two upward-strokes lock", 
    "and another one to unlock",
    "and switch back.",
    ""
};
char* helptxt_modes_p2b[] = {
    "Alternatively, you can click once",
    "or twice on the upper mode-indi-", 
    "cator. It shows a bold 'a'",
    "for lower-case and 'A' for",
    "capital mode.",
    ""
};
char* helptxt_modes_p2c[] = {
    "Quickly tap the text area to", 
    "write punctuations and other", 
    "special chars or use the",
    "middle mode-indicator.",
    ""
};

char* helptxt_modes_p3a[] = {
    "Draw a straight diagonal", 
    "up/right-stroke to switch",
    "to numeric mode.", 
    "Draw the stroke a second", 
    "time to switch back to",
    "letter-mode.",
    ""
};
char* helptxt_modes_p3b[] = {
    "Alternatively, you can use the",
    "lower mode-indicator. It shows",
    "a bold '1' for numeric mode", 
    "and after a second click a '+'",
    "for extended numeric symbols.", 
    ""
};
char* helptxt_modes_p3c[] = {
    "Quickly tap the text area", 
    "(in numeric mode) to write",
    "characters like '+', '{', ...",
    ""
};

char* helptxt_alpha_p2a[] = {
    "Hint: You can", 
    "also write 'y'",
    "like a greek",
    "'gamma'-symbol.", 
    ""
};
char* helptxt_alpha_p2b[] = {
    "In letter and numeric mode you",
    "can write:", 
    ""
};

char* helptxt_alpha_p3a[] = {
    BOLD "Capital letters:", 
    YSPACE3,
    "In capital-mode you can use all",
    "lower-case strokes and:", 
    ""
};
char* helptxt_alpha_p3b[] = {
    "In letter- and",  
    "numeric-mode you",
    "can copy and", 
    "paste via pen-",
    "strokes.",
    ""
};


char* helptxt_num_p1[] = {
    "Draw a diagonal up/right-", 
    "stroke to switch to", 
    "numeric mode.", 
    ""
};

char* helptxt_ext_p1[] = {
    "Quickly tap the text area to",
    "switch to extended-mode.",
    ""
};

char* helptxt_extnum_p1[] = {
    "Quickly tap the text area in",
    "numeric-mode to switch to",
    "extended-numeric-mode.",
    ""
};

char* helptxt_acc_p1a[] = {
    BOLD "Accents and Umlauts:",
    YSPACE3,
    "First write the letter that you",
    "wish to put the accent on.",
    "Then write one of the following",
    "accent-strokes.",
    ""
};
char* helptxt_acc_p1b[] = {
    "Please note that the first", 
    "accent is a straight down-and-",
    "left line that goes back up to",
    "the starting point.",
    "The 2nd and 3rd pattern start", 
    "like the 1st. The 2nd then goes",
    "down-left a second time, while", 
    "the 3rd goes down-right instead.",
    ""
};

char* helptxt_pg_p1[] = {
#ifdef TEST2
    BOLD "Memos, Contacts & Pages:", 
/*    YSPACE3,
    "PVMerlin works directly on the",
    "PV-Memos and PV-Contacts.",
    "There is no problem in using", 
    "both PVMerlin and PV-Memo or",
    "PV-Contacts on the same data.", */
#else
    BOLD "Memos and Pages:", 
    YSPACE3,
    "PVMerlin works directly on the",
    "PV-Memos. There is no problem", 
    "in using both PVMerlin and PV-",
    "Memo on the same data.",
#endif
    YSPACE3,
    "Memos with the same title are",
    "seen as one logical document", 
    "with several pages. ",
    YSPACE3,
    "PgUp/PgDn buttons (above mode", 
    "buttons) skim through the",
    "document's pages. The current",
    "page is always saved before", 
    "the next is loaded.",
    YSPACE3,
    "The [pg]-menu lets you split",
    "pages in two and merge them", 
    "back together again.",
    ""
};

char* helptxt_pg_p2[] = {
    BOLD "Page titles:", 
    YSPACE3,
    "Titles are only compared up to", 
    "the first '-', ':' or '(' char.",
    "Thus you can have subtitles for",
    "your pages, like: 'Food - 1'"
    
#ifdef TEST2
    " etc.",
    "(Titles equal up to the 45th",
    "char are considered equal.)", 
#else
    ", ", 
    "'Food: best', 'Food (cakes)',", 
    "etc.", 
    YSPACE3,
    "Further, titles are only",
    "compared up to the 45th",
    "character. Longer titles are",   
    "considered equal if they differ",
    "only beyond that point.", 
#endif
#ifdef TEST2
/*Save more space, sorry
    YSPACE3,
    YSPACE3,
    BOLD "Contacts & Pages:", 
    YSPACE3,
    "Contacts can obviously not be",
    "split or merged. Still, PgUp/",
    "PgDn buttons skim through",
    "entries for the same name.", */
#endif
    ""
};

char* helptxt_about[] = {
#ifdef TEST2 /* sorry again, have to save more code size... */
    BOLD "PVMerlin V4.0",
    "(under GNU license)",
    YSPACE4,
    YSPACE4,
    "Thanks to:",
    "- Stefan Hellkvist (original", 
    "  char. recognition code)",
    "- Wittawat Yamwong (1st port",
    "  to the PV)",
    "- Marcin Wolinski (helped",
    "  with GUI and file access)",
    YSPACE4,
    YSPACE4,
    "Have fun!",
    "  Christian Ensel",
#else
  #ifdef TEST
    BOLD "PVMerlin Version 3.2",
  #else
    BOLD "PVMerlin Version 3.1",
  #endif
    YSPACE2,
    "The original core parts of the",
    "pattern-recognition algorithms",
    "come from and are copyrighted", 
    "by Stefan Hellkvist.", 
    YSPACE2,
    "Wittawat Yamwong ported",  
    "Stefan's 'xmerlin' to the",
    "Casio Pocket Viewer.", 
    YSPACE2,
    "I (Christian Ensel) improved the",
    "character recognition and wrote", 
    "the pen handling, GUI and memo",
    "access code.",
    YSPACE2,
    "Marcin Wolinski helped with the",
    "GUI and file access.",
    YSPACE3,
    YSPACE3,
    YSPACE4,
    "Thanks to all, have fun!",
#endif
    ""
};


void merlinhelp(t_helpmode m)
{
    LibOpenWindowS( ON, WINX, WINY, 154, 154 );
    switch (m) {
        case HELPMODE_GENERAL:
            printHelpLines( 5, 5, helptxt_general_p1 );
            if (!waitfornextpage())
                break;
            printHelpLines( 5, 5, helptxt_general_p2 );
            /*
            LibPutProStr( IB_PFONT1, 5, 99, "Pressing <Esc> will ask, if you", 152 );
            LibPutProStr( IB_PFONT1, 5,107, "want to save the current (modi-", 152 );
            LibPutProStr( IB_PFONT1, 5,115, "fied) text. Then PVMerlin exits", 152 );
            LibPutProStr( IB_PFONT1, 5,123, "to the PV main menu. So, if you", 152 );
            LibPutProStr( IB_PFONT1, 5,131, "like what you wrote, do not call", 152 );
            LibPutProStr( IB_PFONT1, 5,139, "other applications directly, but", 152 );
            LibPutProStr( IB_PFONT1, 5,147, "press <Esc> first.", 152 ); */
            waitfornextpage();
            break;

        case HELPMODE_MODES:
            printHelpLines( 5, 5, helptxt_modes_p1 );
            /*LibPutProStr( IB_PFONT1,120,145, "more ...", 152 );*/
            if (!waitfornextpage())
                break;

            LibPutProStr( IB_PFONT2, 5,  5, "Capital letters:", 152 );
            printHelpLines( 22, 15, helptxt_modes_p2a );
            drawOnePattern( 9, 13, ALPHA, UP_STROKE_INDEX ); /* UP_STROKE */ 
            drawOnePattern( 6, 39, ALPHA, UP_STROKE_INDEX ); /* UP_STROKE */
            drawOnePattern(14, 39, ALPHA, UP_STROKE_INDEX ); /* UP_STROKE */
            printHelpLines( 5, 71, helptxt_modes_p2b );

            LibPutProStr( IB_PFONT2, 5,114, "Extended Characters:", 152 );
            printHelpLines( 22, 124, helptxt_modes_p2c );
            LibBox( 9, 130, 3, 3, 1);
            if (!waitfornextpage())
                break;

            LibPutProStr( IB_PFONT2, 5,  5, "Numeric-Mode:", 152 );
            printHelpLines( 28, 16, helptxt_modes_p3a );
            drawOnePattern( 2, 20, ALPHA, NUM_STROKE_INDEX ); /* NUM_STROKE */
            printHelpLines( 5, 67, helptxt_modes_p3b );

            LibPutProStr( IB_PFONT2, 5,118, "Extended Numeric Symbols:", 152 );
            printHelpLines( 22, 129, helptxt_modes_p3c );
            LibBox( 9, 130, 3, 3, 1);

            waitfornextpage();
            break;

        case HELPMODE_ALPHA:
            LibPutProStr( IB_PFONT2, 5,  5, "Letters:", 152 );
            drawPatterns( 2, 12, ALPHA, 3, 22 );
            if (!waitfornextpage())
                break;

            drawPatterns( 2, 7, ALPHA, 23, npatterns[ALPHA]-10 );
            printHelpLines( 76, 80, helptxt_alpha_p2a );
            printHelpLines( 5, 116, helptxt_alpha_p2b );
            drawPatterns( 42,120, ALPHA, 0, 2 );
            if (!waitfornextpage())
                break;

            #ifdef POLISH
            LibPutProStr( IB_PFONT2, 5,  5, "Extra Polish letters:", 152 );
            drawPatterns( 2, 12, ALPHA_POLISH, 0, 2 );
            if (!waitfornextpage())
                break;
            #endif
        
            printHelpLines( 5, 5, helptxt_alpha_p3a );
            drawPatterns( 2, 33, ALPHA_CAPITAL, 0, npatterns[ALPHA_CAPITAL]-1 );
            LibPutProStr( IB_PFONT2, 5,110, "Clipboard:", 152 );
            printHelpLines( 72, 116, helptxt_alpha_p3b );
            drawPatterns( 2,116, ALPHA, COPY_STROKE_INDEX, PASTE_STROKE_INDEX );
            waitfornextpage();
            break;

        case HELPMODE_NUM:
            LibPutProStr( IB_PFONT2,  5,  5, "Numeric Mode:", 152 );
            printHelpLines( 28, 16, helptxt_num_p1 );
            drawOnePattern( 2, 16, ALPHA, NUM_STROKE_INDEX ); /* NUM_STROKE */
            drawPatterns( 2, 46, NUMERAL, 3, npatterns[NUMERAL]-5 );
            waitfornextpage();
            break;

        case HELPMODE_EXT:
            LibPutProStr( IB_PFONT2, 5,  5, "Extended Characters:", 152 );
            printHelpLines( 22, 16, helptxt_ext_p1 );
            LibPutProStr( IB_PFONT1, 5, 32, "Tap a second time to write '.'.", 152 );
            LibBox( 9, 20, 3, 3, 1);
            drawPatterns( 2, 46, ALPHA_EXT, 1, 15);
            if (!waitfornextpage())
                break;

            drawPatterns( 2, 12, ALPHA_EXT, 16, npatterns[ALPHA_EXT]-2 );
            waitfornextpage();
            break;

        case HELPMODE_EXT_NUM:
            LibPutProStr( IB_PFONT2, 5,  5, "Extended Numeric Symbols:", 152 );
            printHelpLines( 22, 16, helptxt_extnum_p1 );
            LibBox( 9, 28, 3, 3, 1);
            drawPatterns( 2, 46, NUMERAL_EXT, 1, 15 );
            if (!waitfornextpage())
                break;

            drawPatterns( 2, 12, NUMERAL_EXT, 16, npatterns[NUMERAL_EXT]-2 );
            waitfornextpage();
            break;

        case HELPMODE_ACC:
            printHelpLines( 5, 5, helptxt_acc_p1a );
            drawPatterns( 2, 52, ALPHA, npatterns[ALPHA]-7, npatterns[ALPHA]-3);
            printHelpLines( 5, 90, helptxt_acc_p1b );
            waitfornextpage();
            break;

        case HELPMODE_PAGES:
            printHelpLines( 5, 5, helptxt_pg_p1 );
            if (!waitfornextpage())
                break;

            printHelpLines( 5, 5, helptxt_pg_p2 );
            waitfornextpage();
            break;

        case HELPMODE_ABOUT:
            printHelpLines( 5, 5, helptxt_about );
            LibPutProStr( IB_PFONT1,140,146, "ce", 152 );
            waitfornextpage();
            break;
    }
    LibCloseWindow();    
    LibPutDisp();
}













