/* permanent.h
   2001/07/29 Marcin Woli\'nski

   Permanent storage of PV applets' data.  

   Based on CASIO's 16-puzzle example.  

*/
#include <stdrom.h>

#include "define.h"
#include "libc.h"
#include "permanent.h"

static byte fm_entry; /* File Main Entry */
static byte fs_entry; /* File Sub Entry */
static byte entry_cond; /* Is everything OK? */
/* To have a file >3KB one has to use LFILE_BUF below and respective
   variants of all flash functions: */
static FILE_BUF	fbuf; /* FLASH data buffer (binaryBuffer 3KB)*/
static FILE_INFO finfo; /* FLASH data information */

bool permanentInit( const char* fname, void **data, word dlen ) {
  char fnamebuf[16];
  int fileexisted=TRUE; /*ceV3.1*/
  *data = &fbuf.fbuf.bin.bin_buf;
  fbuf.fbuf.bin.char_num = dlen;	/* BinaryDataCapacity */
  entry_cond = FALSE;
  /* CASIO's docs are somewhat unclear in requiring the file name to
   be 16 bytes long.  To be on the safe side we do the following: */
  memset(fnamebuf, 0, 16);
  strcpy(fnamebuf, fname);
/*ce-new changed:
  if(LibSubEntrySave(fnamebuf,&fs_entry)==FALSE){
    fm_entry = 0xff;
    fs_entry = 0xff;
    return(FALSE);
  }
  LibGetAllEntry(fnamebuf,&fm_entry,&fs_entry);
to: (speeds up loading process) */
  if (LibGetAllEntry(fnamebuf,&fm_entry,&fs_entry)==FALSE) { /* File does not exist, yet */
    memset(*data,0,dlen);
    if (LibSubEntrySave(fnamebuf,&fs_entry)==FALSE) { /* create file */
      fm_entry = 0xff;
      fs_entry = 0xff;
      return(FALSE);
    }
    /*ceV3.1*/
    fileexisted=FALSE;
    if (LibGetAllEntry(fnamebuf,&fm_entry,&fs_entry)==FALSE) { /* File does still not exist! */
      fm_entry = 0xff;
      fs_entry = 0xff;
      return(FALSE);
    }
  }
/*ce-new end*/

  entry_cond = TRUE;
  /*  LibPutMsgDlg("fs_entry: %d\r fm_entry: %d", fs_entry, fm_entry);*/

  fbuf.fsb_main_entry_ = fm_entry;		/* Main */
  fbuf.fsb_sub_entry_  = fs_entry;		/* Sub */
  fbuf.fsb_scrt_info_  = FILE_OPEN_MODE;	/* Open Mode */

  finfo.fp      = 0xffff;		/* first Data */
  finfo.kind    = FILE_KIND_BIN;

  if(LibFileFindNext(&fbuf,&finfo,0x00)==TRUE){
    if (LibFileRead(&fbuf,&finfo) == FALSE) 
      LibPutMsgDlg("Strange error"); /* Read */
    /* This is in case the file doesn't in fact contain the structure
       we want it to:
       [We could as well check if char_num==dlen ?]
    */
    fbuf.fbuf.bin.char_num = dlen;	/* BinaryDataCapacity */
    return(fileexisted);
  }else{
    /*memset(*data,0,dlen); /* ce-new line deleted */
    return(FALSE);
  }
}


void permanentSave() {
  if(entry_cond == FALSE)
    return;
  fbuf.fsb_main_entry_ = fm_entry;			/* Main */
  fbuf.fsb_sub_entry_  = fs_entry;			/* Sub */
  fbuf.fsb_scrt_info_  = FILE_OPEN_MODE;		/* Open Mode */
  
  finfo.fp      = 0xffff;				/* first Data */
  finfo.kind    = FILE_KIND_BIN;			/* Bin */
  
  if(LibFileFindNext(&fbuf,&finfo,0x00)==FALSE){
    finfo.fp  = 0xffff;					/* first Data */
  }
  
  fbuf.fsb_main_entry_ = fm_entry;			/* Main */
  fbuf.fsb_sub_entry_  = fs_entry;			/* Sub */
  fbuf.fsb_scrt_info_  = FILE_OPEN_MODE;		/* Open Mode */
  
  finfo.kind    = FILE_KIND_BIN;			/* Bin */
  
  if(LibFileWrite(&fbuf, &finfo) == FALSE){             /* Saving data */
    LibErrorDisp(FlashStatus);				/* Error Disp */
  }
  
}
